# Copyright (C) 2017 Open Information Security Foundation
# Copyright (c) 2016 Jason Ish
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

import logging
import time

GREEN = "\x1b[32m"
BLUE = "\x1b[34m"
REDB = "\x1b[1;31m"
YELLOW = "\x1b[33m"
RED = "\x1b[31m"
YELLOWB = "\x1b[1;33m"
ORANGE = "\x1b[38;5;208m"
RESET = "\x1b[0m"

# A list of secrets that will be replaced in the log output.
secrets = {}

def add_secret(secret, replacement):
    """Register a secret to be masked. The secret will be replaced with:
           <replacement>
    """
    secrets[str(secret)] = str(replacement)

class SuriColourLogHandler(logging.StreamHandler):
    """An alternative stream log handler that logs with Suricata inspired
    log colours."""

    @staticmethod
    def format_time(record):
        local_time = time.localtime(record.created)
        formatted_time = "%d/%d/%d -- %02d:%02d:%02d" % (local_time.tm_mday,
                                                         local_time.tm_mon,
                                                         local_time.tm_year,
                                                         local_time.tm_hour,
                                                         local_time.tm_min,
                                                         local_time.tm_sec)
        return "%s" % (formatted_time)

    def emit(self, record):

        if record.levelname == "ERROR":
            level_prefix = REDB
            message_prefix = REDB
        elif record.levelname == "WARNING":
            level_prefix = ORANGE
            message_prefix = ORANGE
        else:
            level_prefix = YELLOW
            message_prefix = ""

        self.stream.write("%s%s%s - <%s%s%s> -- %s%s%s\n" % (
            GREEN,
            self.format_time(record),
            RESET,
            level_prefix,
            record.levelname.title(),
            RESET,
            message_prefix,
            self.mask_secrets(record.getMessage()),
            RESET))

    @staticmethod
    def mask_secrets(msg):
        for secret in secrets:
            msg = msg.replace(secret, "<%s>" % secrets[secret])
        return msg



from __future__ import print_function

import unittest

from suricata.ctl import filestore

class PruneTestCase(unittest.TestCase):

    def test_parse_age(self):
        self.assertEqual(filestore.parse_age("1s"), 1)
        self.assertEqual(filestore.parse_age("1m"), 60)
        self.assertEqual(filestore.parse_age("1h"), 3600)
        self.assertEqual(filestore.parse_age("1d"), 86400)

        with self.assertRaises(filestore.InvalidAgeFormatError):
            filestore.parse_age("1")
        with self.assertRaises(filestore.InvalidAgeFormatError):
            filestore.parse_age("1y")



# Copyright (C) 2017 Open Information Security Foundation
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

import unittest

from suricata.update.maps import ClassificationMap

class ClassificationMapTestCase(unittest.TestCase):

    test_filename = "tests/classification1.config"

    def test_load_from_file(self):
        m = ClassificationMap(open(self.test_filename))

        # Classifications are indexed at 1.
        self.assertEqual(None, m.get(0))

        c = m.get(1)
        self.assertEqual("not-suspicious", c["name"])
        self.assertEqual("Not Suspicious Traffic", c["description"])
        self.assertEqual(3, c["priority"])

        c = m.get(34)
        self.assertEqual("default-login-attempt", c["name"])
        self.assertEqual("Attempt to login by a default username and password",
                          c["description"])
        self.assertEqual(2, c["priority"])

        c = m.get_by_name("unknown")
        self.assertTrue(c is not None)
        self.assertEqual("unknown", c["name"])
        self.assertEqual("Unknown Traffic", c["description"])
        self.assertEqual(3, c["priority"])



# Copyright (C) 2018 Open Information Security Foundation
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

from __future__ import print_function

import sys
import os
import os.path
import time
import re
import logging

logger = logging.getLogger("filestore")


class InvalidAgeFormatError(Exception):
    pass


def register_args(parser):
    subparser = parser.add_subparsers(help="sub-command help")
    prune_parser = subparser.add_parser("prune",
            help="Remove files in specified directory older than specified age")
    required_args = prune_parser.add_argument_group("required arguments")
    required_args.add_argument("-d", "--directory",
            help="filestore directory", required=True)
    required_args.add_argument("--age",
            help="prune files older than age, units: s, m, h, d")
    prune_parser.add_argument(
        "-n", "--dry-run", action="store_true", default=False,
        help="only print what would happen")
    prune_parser.add_argument(
        "-v", "--verbose", action="store_true",
        default=False, help="increase verbosity")
    prune_parser.add_argument(
        "-q", "--quiet", action="store_true", default=False,
        help="be quiet, log warnings and errors only")
    prune_parser.set_defaults(func=prune)


def is_fileinfo(path):
    return path.endswith(".json")


def parse_age(age):
    matched_age = re.match(r"(\d+)\s*(\w+)", age)
    if not matched_age:
        raise InvalidAgeFormatError(age)
    val = int(matched_age.group(1))
    unit = matched_age.group(2)
    ts_units = ["s", "m", "h", "d"]
    try:
        idx = ts_units.index(unit)
    except ValueError:
        raise InvalidAgeFormatError("bad unit: %s" % (unit))
    multiplier = 60 ** idx if idx != 3 else 24 * 60 ** 2
    return val * multiplier


def get_filesize(path):
    return os.stat(path).st_size


def remove_file(path, dry_run):
    size = 0
    size += get_filesize(path)
    if not dry_run:
        os.unlink(path)
    return size


def set_logger_level(args):
    if args.verbose:
        logger.setLevel(logging.DEBUG)
    if args.quiet:
        logger.setLevel(logging.WARNING)


def perform_sanity_checks(args):
    set_logger_level(args)
    err_msg = {
            "directory": "filestore directory must be provided",
            "age": "no age provided, nothing to do",
            }
    for val, msg in err_msg.items():
        if not getattr(args, val):
            print("Error: {}".format(msg), file=sys.stderr)
            sys.exit(1)
    required_dirs = ["tmp", "00", "ff"]
    for required_dir in required_dirs:
        if not os.path.exists(os.path.join(args.directory, required_dir)):
            logger.error("Provided directory is not a filestore directory")
            sys.exit(1)


def prune(args):
    perform_sanity_checks(args)
    age = parse_age(args.age)
    now = time.time()
    size = 0
    count = 0

    for dirpath, dirnames, filenames in os.walk(args.directory, topdown=True):
        # Do not go into the tmp directory.
        if "tmp" in dirnames:
            dirnames.remove("tmp")
        for filename in filenames:
            path = os.path.join(dirpath, filename)
            mtime = os.path.getmtime(path)
            this_age = now - mtime
            if this_age > age:
                logger.debug("Deleting %s; age=%ds", path, this_age)
                size += remove_file(path, args.dry_run)
                count += 1
    logger.info("Removed %d files; %d bytes.", count, size)
    return 0



import sys

result = []
cur_count = 0
cur_val = 0
for byte in sys.stdin.read():
    if byte == '1':
        cur_val |= (1<<cur_count)
    elif byte != '0':
        break
    cur_count += 1
    if cur_count == 8:
        result.append(chr(cur_val))
        cur_val = 0
        cur_count = 0
if cur_count != 0:
    result.append(chr(cur_val))
sys.stdout.write(''.join(result))



#!/usr/bin/python
#I love the python Power Glove. It's so bad!
#Usage: sudo -u suricata ./sock_to_gzip_file.py --output-file="http.log.gz" --listen-sock="http.log.sock"

import socket,os
import gzip
import sys
from optparse import OptionParser

if __name__ == "__main__":
    parser = OptionParser()
    #Path to the socket
    parser.add_option("--listen-sock", dest="lsock", type="string", help="Path to the socket we will listen on.")
    #Path to gzip file we will write
    parser.add_option("--output-file", dest="output", type="string", help="Path to file name to output gzip file we will write to.")

    #parse the opts
    (options, args) = parser.parse_args()

    options.usage = "example: sudo -u suricata ./sock_to_gzip_file.py --output-file=\"http.log.gz\" --listen-sock=\"http.log.sock\"\n"
    #Open the output file
    if options.output:
        try:
            f = gzip.open(options.output, 'wb')
        except Exception,e:
            print("Error: could not open output file %s:\n%s\n", options.output, e)
            sys.exit(-1)
    else:
        print("Error: --output-file option required and was not specified\n%s" % (options.usage))
        sys.exit(-1)

    #Open our socket and bind
    if options.lsock:
        if os.path.exists(options.lsock):
            try:
                os.remove(options.lsock)
            except OSError:
                pass
        try:
            s = socket.socket(socket.AF_UNIX, socket.SOCK_STREAM)
            s.bind(options.lsock)
            s.listen(1)
            conn, addr = s.accept()
        except Exception,e:
            print("Error: Failed to bind socket %s\n%s\n", options.lsock, e)
            sys.exit(-1)
    else:
        print("Error: --listen-sock option required and was not specified\n%s" % (options.usage))
        sys.exit(-1)

    #Read data from the socket and write to the file
    while 1:
        data = conn.recv(1024)
        if not data: break
        f.write(data)
    conn.close()
    f.close()



# Copyright(C) 2012-2023 Open Information Security Foundation

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, version 2 of the License.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

try:
    import simplejson as json
except ImportError:
    import json
import readline
import select
import sys
from socket import AF_UNIX, error, socket
from inspect import currentframe

from suricata.sc.specs import argsd

SURICATASC_VERSION = "1.0"
VERSION = "0.2"
INC_SIZE = 1024


def get_linenumber():
    cf = currentframe()
    return cf.f_back.f_lineno


class SuricataException(Exception):
    """
    Generic class for suricatasc exception
    """
    def __init__(self, value):
        super(SuricataException, self).__init__(value)
        self.value = value

    def __str__(self):
        return str(self.value)


class SuricataNetException(SuricataException):
    """
    Exception raised when a network error occurs
    """


class SuricataCommandException(SuricataException):
    """
    Exception raised when the command is incorrect
    """


class SuricataReturnException(SuricataException):
    """
    Exception raised when return message is incorrect
    """


class SuricataCompleter:
    def __init__(self, words):
        self.words = words
        self.generator = None

    def complete(self, text):
        for word in self.words:
            if word.startswith(text):
                yield word

    def __call__(self, text, state):
        if state == 0:
            self.generator = self.complete(text)
        try:
            return next(self.generator)
        except StopIteration:
            return None


class SuricataSC:
    def __init__(self, sck_path, verbose=False):
        self.basic_commands = [
                "shutdown",
                "quit",
                "pcap-file-number",
                "pcap-file-list",
                "pcap-last-processed",
                "pcap-interrupt",
                "iface-list",
                "reload-tenants",
                ]
        self.fn_commands = [
                "pcap-file",
                "pcap-file-continuous",
                "iface-stat",
                "conf-get",
                "unregister-tenant-handler",
                "register-tenant-handler",
                "unregister-tenant",
                "register-tenant",
                "reload-tenant",
                "add-hostbit",
                "remove-hostbit",
                "list-hostbit",
                "memcap-set",
                "memcap-show",
                "dataset-add",
                "dataset-remove",
                "get-flow-stats-by-id",
                "dataset-clear",
                "dataset-lookup",
                ]
        self.cmd_list = self.basic_commands + self.fn_commands
        self.sck_path = sck_path
        self.verbose = verbose
        self.socket = socket(AF_UNIX)

    def json_recv(self):
        cmdret = None
        data = ""
        while True:
            if sys.version < '3':
                received = self.socket.recv(INC_SIZE)
            else:
                received = self.socket.recv(INC_SIZE).decode('iso-8859-1')

            if not received:
                break

            data += received
            if data.endswith('\n'):
                cmdret = json.loads(data)
                break
        return cmdret

    def send_command(self, command, arguments=None):
        if command not in self.cmd_list and command != 'command-list':
            raise SuricataCommandException("L{}: Command not found: {}".format(get_linenumber(), command))

        cmdmsg = {}
        cmdmsg['command'] = command
        if arguments:
            cmdmsg['arguments'] = arguments
        if self.verbose:
            print("SND: " + json.dumps(cmdmsg))
        cmdmsg_str = json.dumps(cmdmsg) + "\n"
        if sys.version < '3':
            self.socket.send(cmdmsg_str)
        else:
            self.socket.send(bytes(cmdmsg_str, 'iso-8859-1'))

        ready = select.select([self.socket], [], [], 600)
        if ready[0]:
            cmdret = self.json_recv()
        else:
            cmdret = None
        if not cmdret:
            raise SuricataReturnException("L{}: Unable to get message from server".format(get_linenumber))

        if self.verbose:
            print("RCV: "+ json.dumps(cmdret))

        return cmdret

    def connect(self):
        try:
            if self.socket is None:
                self.socket = socket(AF_UNIX)
            self.socket.connect(self.sck_path)
        except error as err:
            raise SuricataNetException("L{}: {}".format(get_linenumber(), err))

        self.socket.settimeout(10)
        #send version
        if self.verbose:
            print("SND: " + json.dumps({"version": VERSION}))
        if sys.version < '3':
            self.socket.send(json.dumps({"version": VERSION}))
        else:
            self.socket.send(bytes(json.dumps({"version": VERSION}), 'iso-8859-1'))

        ready = select.select([self.socket], [], [], 600)
        if ready[0]:
            cmdret = self.json_recv()
        else:
            cmdret = None

        if not cmdret:
            raise SuricataReturnException("L{}: Unable to get message from server".format(get_linenumber()))

        if self.verbose:
            print("RCV: "+ json.dumps(cmdret))

        if cmdret["return"] == "NOK":
            raise SuricataReturnException("L{}: Error: {}".format(get_linenumber(), cmdret["message"]))

        cmdret = self.send_command("command-list")

        # we silently ignore NOK as this means server is old
        if cmdret["return"] == "OK":
            self.cmd_list = cmdret["message"]["commands"]
            self.cmd_list.append("quit")

    def close(self):
        self.socket.close()
        self.socket = None

    def execute(self, command):
        full_cmd = command.split()
        cmd = full_cmd[0]
        cmd_specs = argsd[cmd]
        required_args_count = len([d["required"] for d in cmd_specs if d["required"] and not "val" in d])
        arguments = dict()
        for c, spec in enumerate(cmd_specs, 1):
            spec_type = str if "type" not in spec else spec["type"]
            if spec["required"]:
                if spec.get("val"):
                    arguments[spec["name"]] = spec_type(spec["val"])
                    continue
                try:
                    arguments[spec["name"]] = spec_type(full_cmd[c])
                except IndexError:
                    phrase = " at least" if required_args_count != len(cmd_specs) else ""
                    msg = "Missing arguments: expected{} {}".format(phrase, required_args_count)
                    raise SuricataCommandException("L{}: {}".format(get_linenumber(), msg))
                except ValueError as ve:
                    raise SuricataCommandException("L{}: Erroneous arguments: {}".format(get_linenumber(), ve))
            elif c < len(full_cmd):
                arguments[spec["name"]] = spec_type(full_cmd[c])
        return cmd, arguments

    def parse_command(self, command):
        arguments = None
        cmd = command.split()[0] if command else None
        if cmd in self.cmd_list:
            if cmd in self.fn_commands:
                cmd, arguments = getattr(self, "execute")(command=command)
        else:
            raise SuricataCommandException("L{}: Unknown command: {}".format(get_linenumber(), command))
        return cmd, arguments

    def interactive(self):
        print("Command list: " + ", ".join(self.cmd_list))
        try:
            readline.set_completer(SuricataCompleter(self.cmd_list))
            readline.set_completer_delims(";")
            readline.parse_and_bind('tab: complete')
            while True:
                if sys.version < '3':
                    command = raw_input(">>> ").strip()
                else:
                    command = input(">>> ").strip()
                if command == "quit":
                    break
                if len(command.strip()) == 0:
                    continue
                try:
                    cmd, arguments = self.parse_command(command)
                except SuricataCommandException as err:
                    print(err)
                    continue
                try:
                    cmdret = self.send_command(cmd, arguments)
                except IOError as err:
                    # try to reconnect and resend command
                    print("Connection lost, trying to reconnect")
                    try:
                        self.close()
                        self.connect()
                    except (SuricataNetException, SuricataReturnException) as err:
                        print(err.value)
                        continue
                    cmdret = self.send_command(cmd, arguments)
                except (SuricataCommandException, SuricataReturnException) as err:
                    print("An exception occured: " + str(err.value))
                    continue
                #decode json message
                if cmdret["return"] == "NOK":
                    print("Error:")
                    print(json.dumps(cmdret["message"], sort_keys=True, indent=4, separators=(',', ': ')))
                else:
                    print("Success:")
                    print(json.dumps(cmdret["message"], sort_keys=True, indent=4, separators=(',', ': ')))
        except KeyboardInterrupt:
            print("[!] Interrupted")
            sys.exit(0)



# Copyright (C) 2017 Open Information Security Foundation
# Copyright (c) 2017 Jason Ish
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

from __future__ import print_function

import tarfile
from zipfile import ZipFile

def extract_tar(filename):
    files = {}

    tf = tarfile.open(filename, mode="r:*")

    try:
        while True:
            member = tf.next()
            if member is None:
                break
            if not member.isfile():
                continue
            fileobj = tf.extractfile(member)
            if fileobj:
                # Remove leading /.
                member_name = member.name.lstrip("/")
                files[member_name] = fileobj.read()
    finally:
        tf.close()

    return files

def extract_zip(filename):
    files = {}

    with ZipFile(filename) as reader:
        for name in reader.namelist():
            if name.endswith("/"):
                continue
            fixed_name = name.lstrip("/")
            files[fixed_name] = reader.read(name)
    
    return files

def try_extract(filename):
    try:
        return extract_tar(filename)
    except:
        pass

    try:
        return extract_zip(filename)
    except:
        pass
    
    return None



import sys
result = []
def reverse(x):
    return x[::-1]
    ret = ''
    for y in reversed(x):
        ret += y
    return ret
for byte in sys.stdin.read():
    result.append(reverse("{0:#010b}".format(ord(byte)).replace("0b","")))
print ''.join(result)



index = {   'sources': {   'abuse.ch/feodotracker': {   'checksum': False,
                                                'description': 'The Suricata '
                                                               'Botnet C2 IP '
                                                               'Ruleset '
                                                               'contains '
                                                               'botnet C2s '
                                                               'tracked by\n'
                                                               'Feodo Tracker '
                                                               'and can be '
                                                               'used for both, '
                                                               'Suricata and '
                                                               'Snort open\n'
                                                               'source '
                                                               'IDS/IPS. If '
                                                               'you are '
                                                               'running '
                                                               'Suricata or '
                                                               'Snort, you '
                                                               'can\n'
                                                               'use this '
                                                               'ruleset to '
                                                               'detect and/or '
                                                               'block network '
                                                               'connections\n'
                                                               'towards '
                                                               'hostline '
                                                               'servers (IP '
                                                               'address:port '
                                                               'combination).\n',
                                                'license': 'CC0-1.0',
                                                'summary': 'Abuse.ch Feodo '
                                                           'Tracker Botnet C2 '
                                                           'IP ruleset',
                                                'url': 'https://feodotracker.abuse.ch/downloads/feodotracker.tar.gz',
                                                'vendor': 'Abuse.ch'},
                   'abuse.ch/sslbl-blacklist': {   'checksum': False,
                                                   'description': 'The SSL '
                                                                  'Blacklist '
                                                                  '(SSLBL) is '
                                                                  'a project '
                                                                  'of abuse.ch '
                                                                  'with the '
                                                                  'goal\n'
                                                                  'of '
                                                                  'detecting '
                                                                  'malicious '
                                                                  'SSL '
                                                                  'connections, '
                                                                  'by '
                                                                  'identifying '
                                                                  'and\n'
                                                                  'blacklisting '
                                                                  'SSL '
                                                                  'certificates '
                                                                  'used by '
                                                                  'botnet C&C '
                                                                  'servers. '
                                                                  'In\n'
                                                                  'addition, '
                                                                  'SSLBL '
                                                                  'identifies '
                                                                  'JA3 '
                                                                  'fingerprints '
                                                                  'that helps '
                                                                  'you to\n'
                                                                  'detect & '
                                                                  'block '
                                                                  'malware '
                                                                  'botnet C&C '
                                                                  'communication '
                                                                  'on the TCP\n'
                                                                  'layer.\n',
                                                   'license': 'CC0-1.0',
                                                   'replaces': [   'sslbl/ssl-fp-blacklist'],
                                                   'summary': 'Abuse.ch SSL '
                                                              'Blacklist',
                                                   'url': 'https://sslbl.abuse.ch/blacklist/sslblacklist_tls_cert.tar.gz',
                                                   'vendor': 'Abuse.ch'},
                   'abuse.ch/sslbl-c2': {   'checksum': False,
                                            'description': 'This ruleset '
                                                           'contains all '
                                                           'botnet '
                                                           'Command&Control '
                                                           'servers (C&Cs)\n'
                                                           'identified by '
                                                           'SSLBL to be '
                                                           'associated with a '
                                                           'blacklisted SSL\n'
                                                           'certificate.\n',
                                            'license': 'CC0-1.0',
                                            'summary': 'Abuse.ch Suricata '
                                                       'Botnet C2 IP Ruleset',
                                            'url': 'https://sslbl.abuse.ch/blacklist/sslipblacklist.tar.gz',
                                            'vendor': 'Abuse.ch'},
                   'abuse.ch/sslbl-ja3': {   'checksum': False,
                                             'description': 'If you are '
                                                            'running Suricata, '
                                                            'you can use the '
                                                            "SSLBL's Suricata\n"
                                                            'JA3 fingerprint '
                                                            'ruleset to detect '
                                                            'and/or block '
                                                            'malicious SSL\n'
                                                            'connections in '
                                                            'your network '
                                                            'based on the JA3 '
                                                            'fingerprint. '
                                                            'Please\n'
                                                            'note that your '
                                                            'need Suricata '
                                                            '4.1.0 or newer in '
                                                            'order to use the\n'
                                                            'JA3 fingerprint '
                                                            'ruleset.\n',
                                             'license': 'CC0-1.0',
                                             'min-version': '4.1.0',
                                             'replaces': [   'sslbl/ja3-fingerprints'],
                                             'summary': 'Abuse.ch Suricata JA3 '
                                                        'Fingerprint Ruleset',
                                             'url': 'https://sslbl.abuse.ch/blacklist/ja3_fingerprints.tar.gz',
                                             'vendor': 'Abuse.ch'},
                   'abuse.ch/urlhaus': {   'checksum': False,
                                           'description': 'URLhaus is a '
                                                          'project from '
                                                          'abuse.ch with the '
                                                          'goal of sharing\n'
                                                          'malicious URLs that '
                                                          'are being used for '
                                                          'malware '
                                                          'distribution.\n',
                                           'license': 'CC0-1.0',
                                           'summary': 'Abuse.ch URLhaus '
                                                      'Suricata Rules',
                                           'url': 'https://urlhaus.abuse.ch/downloads/urlhaus_suricata.tar.gz',
                                           'vendor': 'abuse.ch'},
                   'aleksibovellan/nmap': {   'checksum': False,
                                              'description': 'These detection '
                                                             'rules work by '
                                                             'looking for '
                                                             'specific NMAP\n'
                                                             'packet window '
                                                             'sizes, flags, '
                                                             'port numbers, '
                                                             'and known NMAP\n'
                                                             'timing '
                                                             'intervals.\n',
                                              'homepage': 'https://github.com/aleksibovellan/opnsense-suricata-nmaps',
                                              'license': 'MIT',
                                              'min-version': '7.0.4',
                                              'summary': 'Suricata IDS/IPS '
                                                         'Detection Rules '
                                                         'Against NMAP Scans',
                                              'url': 'https://raw.githubusercontent.com/aleksibovellan/opnsense-suricata-nmaps/main/local.rules',
                                              'vendor': 'aleksibovellan'},
                   'et/open': {   'description': 'Proofpoint ET Open is a '
                                                 'timely and accurate rule set '
                                                 'for detecting and blocking '
                                                 'advanced threats\n',
                                  'license': 'MIT',
                                  'summary': 'Emerging Threats Open Ruleset',
                                  'url': 'https://rules.emergingthreats.net/open/suricata-%(__version__)s/emerging.rules.tar.gz',
                                  'vendor': 'Proofpoint'},
                   'et/pro': {   'checksum': False,
                                 'description': 'Proofpoint ET Pro is a timely '
                                                'and accurate rule set for '
                                                'detecting and blocking '
                                                'advanced threats\n',
                                 'license': 'Commercial',
                                 'parameters': {   'secret-code': {   'prompt': 'Emerging '
                                                                                'Threats '
                                                                                'Pro '
                                                                                'access '
                                                                                'code'}},
                                 'replaces': ['et/open'],
                                 'subscribe-url': 'https://www.proofpoint.com/us/threat-insight/et-pro-ruleset',
                                 'summary': 'Emerging Threats Pro Ruleset',
                                 'url': 'https://rules.emergingthreatspro.com/%(secret-code)s/suricata-%(__version__)s/etpro.rules.tar.gz',
                                 'vendor': 'Proofpoint'},
                   'etnetera/aggressive': {   'checksum': False,
                                              'license': 'MIT',
                                              'min-version': '4.0.0',
                                              'summary': 'Etnetera aggressive '
                                                         'IP blacklist',
                                              'url': 'https://security.etnetera.cz/feeds/etn_aggressive.rules',
                                              'vendor': 'Etnetera a.s.'},
                   'malsilo/win-malware': {   'checksum': True,
                                              'description': 'TCP/UDP, DNS and '
                                                             'HTTP Windows '
                                                             'threats '
                                                             'artifacts '
                                                             'observed at '
                                                             'runtime.\n',
                                              'homepage': 'https://raw-data.gitlab.io/post/malsilo_2.1/',
                                              'license': 'MIT',
                                              'min-version': '4.1.0',
                                              'obsolete': 'unmaintained',
                                              'summary': 'Commodity malware '
                                                         'rules',
                                              'url': 'https://malsilo.gitlab.io/feeds/dumps/malsilo.rules.tar.gz',
                                              'vendor': 'malsilo'},
                   'oisf/trafficid': {   'checksum': False,
                                         'license': 'MIT',
                                         'min-version': '4.0.0',
                                         'summary': 'Suricata Traffic ID '
                                                    'ruleset',
                                         'support-url': 'https://redmine.openinfosecfoundation.org/',
                                         'url': 'https://openinfosecfoundation.org/rules/trafficid/trafficid.rules',
                                         'vendor': 'OISF'},
                   'pawpatrules': {   'checksum': False,
                                      'description': 'PAW Patrules ruleset '
                                                     'permit to detect many '
                                                     'events on\n'
                                                     'network. Suspicious '
                                                     'flow, malicious tool, '
                                                     'unsuported and\n'
                                                     'vulnerable system, known '
                                                     'threat actors with '
                                                     'various IOCs,\n'
                                                     'lateral movement, bad '
                                                     'practice, shadow IT... '
                                                     'Rules are\n'
                                                     'frequently updated.\n',
                                      'homepage': 'https://pawpatrules.fr/',
                                      'license': 'CC-BY-SA-4.0',
                                      'min-version': '7.0.3',
                                      'summary': 'PAW Patrules is a collection '
                                                 'of rules for IDPS / NSM '
                                                 'Suricata engine',
                                      'url': 'https://rules.pawpatrules.fr/suricata/paw-patrules.tar.gz',
                                      'vendor': 'pawpatrules'},
                   'ptresearch/attackdetection': {   'description': 'The '
                                                                    'Attack '
                                                                    'Detection '
                                                                    'Team '
                                                                    'searches '
                                                                    'for new '
                                                                    'vulnerabilities '
                                                                    'and '
                                                                    '0-days, '
                                                                    'reproduces '
                                                                    'it and '
                                                                    'creates '
                                                                    'PoC '
                                                                    'exploits '
                                                                    'to '
                                                                    'understand '
                                                                    'how these '
                                                                    'security '
                                                                    'flaws '
                                                                    'work and '
                                                                    'how '
                                                                    'related '
                                                                    'attacks '
                                                                    'can be '
                                                                    'detected '
                                                                    'on the '
                                                                    'network '
                                                                    'layer. '
                                                                    'Additionally, '
                                                                    'we are '
                                                                    'interested '
                                                                    'in '
                                                                    'malware '
                                                                    'and '
                                                                    "hackers' "
                                                                    'TTPs, so '
                                                                    'we '
                                                                    'develop '
                                                                    'Suricata '
                                                                    'rules for '
                                                                    'detecting '
                                                                    'all sorts '
                                                                    'of such '
                                                                    'activities.\n',
                                                     'license': 'Custom',
                                                     'license-url': 'https://raw.githubusercontent.com/ptresearch/AttackDetection/master/LICENSE',
                                                     'obsolete': 'no longer '
                                                                 'exists',
                                                     'summary': 'Positive '
                                                                'Technologies '
                                                                'Attack '
                                                                'Detection '
                                                                'Team ruleset',
                                                     'url': 'https://raw.githubusercontent.com/ptresearch/AttackDetection/master/pt.rules.tar.gz',
                                                     'vendor': 'Positive '
                                                               'Technologies'},
                   'ptrules/open': {   'description': 'PT\xa0Rules, an\xa0'
                                                      'open-source project '
                                                      'focused on\xa0enhancing '
                                                      'network security '
                                                      'through proactive '
                                                      'threat detection. As\xa0'
                                                      'the PT\xa0Expert '
                                                      'Security Center attack '
                                                      'detection team, we\xa0'
                                                      'are a\xa0dedicated '
                                                      'group of\xa0'
                                                      'cybersecurity experts '
                                                      'committed to\xa0improve '
                                                      'network security '
                                                      'through open-source '
                                                      'initiatives.\n',
                                       'homepage': 'https://rules.ptsecurity.com',
                                       'license': 'Custom',
                                       'license-url': 'https://rules.ptsecurity.com/files/LICENSE.txt',
                                       'min-version': '5.0.0',
                                       'summary': 'Positive Technologies Open '
                                                  'Ruleset',
                                       'url': 'https://rules.ptsecurity.com/files/ptopen.rules.tar.gz',
                                       'vendor': 'Positive Technologies'},
                   'scwx/enhanced': {   'description': 'Broad ruleset composed '
                                                       'of malware rules and '
                                                       'other security-related '
                                                       'countermeasures, and '
                                                       'curated by the '
                                                       'Secureworks Counter '
                                                       'Threat Unit research '
                                                       'team.  This ruleset '
                                                       'has been enhanced with '
                                                       'comprehensive and '
                                                       'fully '
                                                       'standard-compliant '
                                                       'BETTER metadata '
                                                       '(https://better-schema.readthedocs.io/).\n',
                                        'license': 'Commercial',
                                        'min-version': '3.0.0',
                                        'parameters': {   'secret-code': {   'prompt': 'Secureworks '
                                                                                       'Threat '
                                                                                       'Intelligence '
                                                                                       'Authentication '
                                                                                       'Token'}},
                                        'subscribe-url': 'https://www.secureworks.com/contact/ '
                                                         '(Please reference '
                                                         'CTU Countermeasures)',
                                        'summary': 'Secureworks '
                                                   'suricata-enhanced ruleset',
                                        'url': 'https://ws.secureworks.com/ti/ruleset/%(secret-code)s/Suricata_suricata-enhanced_latest.tgz',
                                        'vendor': 'Secureworks'},
                   'scwx/malware': {   'description': 'High-fidelity, '
                                                      'high-priority ruleset '
                                                      'composed mainly of '
                                                      'malware-related '
                                                      'countermeasures and '
                                                      'curated by the '
                                                      'Secureworks Counter '
                                                      'Threat Unit research '
                                                      'team.\n',
                                       'license': 'Commercial',
                                       'min-version': '3.0.0',
                                       'parameters': {   'secret-code': {   'prompt': 'Secureworks '
                                                                                      'Threat '
                                                                                      'Intelligence '
                                                                                      'Authentication '
                                                                                      'Token'}},
                                       'subscribe-url': 'https://www.secureworks.com/contact/ '
                                                        '(Please reference CTU '
                                                        'Countermeasures)',
                                       'summary': 'Secureworks '
                                                  'suricata-malware ruleset',
                                       'url': 'https://ws.secureworks.com/ti/ruleset/%(secret-code)s/Suricata_suricata-malware_latest.tgz',
                                       'vendor': 'Secureworks'},
                   'scwx/security': {   'description': 'Broad ruleset composed '
                                                       'of malware rules and '
                                                       'other security-related '
                                                       'countermeasures, and '
                                                       'curated by the '
                                                       'Secureworks Counter '
                                                       'Threat Unit research '
                                                       'team.\n',
                                        'license': 'Commercial',
                                        'min-version': '3.0.0',
                                        'parameters': {   'secret-code': {   'prompt': 'Secureworks '
                                                                                       'Threat '
                                                                                       'Intelligence '
                                                                                       'Authentication '
                                                                                       'Token'}},
                                        'subscribe-url': 'https://www.secureworks.com/contact/ '
                                                         '(Please reference '
                                                         'CTU Countermeasures)',
                                        'summary': 'Secureworks '
                                                   'suricata-security ruleset',
                                        'url': 'https://ws.secureworks.com/ti/ruleset/%(secret-code)s/Suricata_suricata-security_latest.tgz',
                                        'vendor': 'Secureworks'},
                   'sslbl/ja3-fingerprints': {   'checksum': False,
                                                 'deprecated': 'Renamed to '
                                                               'abuse.ch/sslbl-ja3',
                                                 'description': 'If you are '
                                                                'running '
                                                                'Suricata, you '
                                                                'can use the '
                                                                "SSLBL's "
                                                                'Suricata\n'
                                                                'JA3 '
                                                                'fingerprint '
                                                                'ruleset to '
                                                                'detect and/or '
                                                                'block '
                                                                'malicious '
                                                                'SSL\n'
                                                                'connections '
                                                                'in your '
                                                                'network based '
                                                                'on the JA3 '
                                                                'fingerprint. '
                                                                'Please\n'
                                                                'note that '
                                                                'your need '
                                                                'Suricata '
                                                                '4.1.0 or '
                                                                'newer in '
                                                                'order to use '
                                                                'the\n'
                                                                'JA3 '
                                                                'fingerprint '
                                                                'ruleset.\n',
                                                 'license': 'CC0-1.0',
                                                 'min-version': '4.1.0',
                                                 'summary': 'Abuse.ch Suricata '
                                                            'JA3 Fingerprint '
                                                            'Ruleset',
                                                 'url': 'https://sslbl.abuse.ch/blacklist/ja3_fingerprints.tar.gz',
                                                 'vendor': 'Abuse.ch'},
                   'sslbl/ssl-fp-blacklist': {   'checksum': False,
                                                 'deprecated': 'Renamed to '
                                                               'abuse.ch/sslbl-blacklist',
                                                 'description': 'The SSL '
                                                                'Blacklist '
                                                                '(SSLBL) is a '
                                                                'project of '
                                                                'abuse.ch with '
                                                                'the goal\n'
                                                                'of detecting '
                                                                'malicious SSL '
                                                                'connections, '
                                                                'by '
                                                                'identifying '
                                                                'and\n'
                                                                'blacklisting '
                                                                'SSL '
                                                                'certificates '
                                                                'used by '
                                                                'botnet C&C '
                                                                'servers. In\n'
                                                                'addition, '
                                                                'SSLBL '
                                                                'identifies '
                                                                'JA3 '
                                                                'fingerprints '
                                                                'that helps '
                                                                'you to\n'
                                                                'detect & '
                                                                'block malware '
                                                                'botnet C&C '
                                                                'communication '
                                                                'on the TCP\n'
                                                                'layer.\n',
                                                 'license': 'CC0-1.0',
                                                 'summary': 'Abuse.ch SSL '
                                                            'Blacklist',
                                                 'url': 'https://sslbl.abuse.ch/blacklist/sslblacklist_tls_cert.tar.gz',
                                                 'vendor': 'Abuse.ch'},
                   'stamus/lateral': {   'description': 'Suricata ruleset '
                                                        'specifically focused '
                                                        'on detecting lateral\n'
                                                        'movement in Microsoft '
                                                        'Windows environments '
                                                        'by Stamus Networks\n',
                                         'license': 'GPL-3.0-only',
                                         'min-version': '6.0.6',
                                         'summary': 'Lateral movement rules',
                                         'support-url': 'https://discord.com/channels/911231224448712714/911238451842666546',
                                         'url': 'https://ti.stamus-networks.io/open/stamus-lateral-rules.tar.gz',
                                         'vendor': 'Stamus Networks'},
                   'stamus/nrd-14-open': {   'description': 'Newly Registered '
                                                            'Domains list '
                                                            '(last 14 days) to '
                                                            'match on DNS, TLS '
                                                            'and HTTP '
                                                            'communication.\n'
                                                            'Produced by '
                                                            'Stamus Labs '
                                                            'research team.\n',
                                             'license': 'Commercial',
                                             'min-version': '6.0.0',
                                             'parameters': {   'secret-code': {   'prompt': 'Stamus '
                                                                                            'Networks '
                                                                                            'License '
                                                                                            'code'}},
                                             'subscribe-url': 'https://www.stamus-networks.com/stamus-labs/subscribe-to-threat-intel-feed',
                                             'summary': 'Newly Registered '
                                                        'Domains Open only - '
                                                        '14 day list, complete',
                                             'url': 'https://ti.stamus-networks.io/%(secret-code)s/sti-domains-nrd-14.tar.gz',
                                             'vendor': 'Stamus Networks'},
                   'stamus/nrd-30-open': {   'description': 'Newly Registered '
                                                            'Domains list '
                                                            '(last 30 days) to '
                                                            'match on DNS, TLS '
                                                            'and HTTP '
                                                            'communication.\n'
                                                            'Produced by '
                                                            'Stamus Labs '
                                                            'research team.\n',
                                             'license': 'Commercial',
                                             'min-version': '6.0.0',
                                             'parameters': {   'secret-code': {   'prompt': 'Stamus '
                                                                                            'Networks '
                                                                                            'License '
                                                                                            'code'}},
                                             'subscribe-url': 'https://www.stamus-networks.com/stamus-labs/subscribe-to-threat-intel-feed',
                                             'summary': 'Newly Registered '
                                                        'Domains Open only - '
                                                        '30 day list, complete',
                                             'url': 'https://ti.stamus-networks.io/%(secret-code)s/sti-domains-nrd-30.tar.gz',
                                             'vendor': 'Stamus Networks'},
                   'stamus/nrd-entropy-14-open': {   'description': 'Suspicious '
                                                                    'Newly '
                                                                    'Registered '
                                                                    'Domains '
                                                                    'list with '
                                                                    'high '
                                                                    'entropy '
                                                                    '(last 14 '
                                                                    'days) to '
                                                                    'match on '
                                                                    'DNS, TLS '
                                                                    'and HTTP '
                                                                    'communication.\n'
                                                                    'Produced '
                                                                    'by Stamus '
                                                                    'Labs '
                                                                    'research '
                                                                    'team.\n',
                                                     'license': 'Commercial',
                                                     'min-version': '6.0.0',
                                                     'parameters': {   'secret-code': {   'prompt': 'Stamus '
                                                                                                    'Networks '
                                                                                                    'License '
                                                                                                    'code'}},
                                                     'subscribe-url': 'https://www.stamus-networks.com/stamus-labs/subscribe-to-threat-intel-feed',
                                                     'summary': 'Newly '
                                                                'Registered '
                                                                'Domains Open '
                                                                'only - 14 day '
                                                                'list, high '
                                                                'entropy',
                                                     'url': 'https://ti.stamus-networks.io/%(secret-code)s/sti-domains-entropy-14.tar.gz',
                                                     'vendor': 'Stamus '
                                                               'Networks'},
                   'stamus/nrd-entropy-30-open': {   'description': 'Suspicious '
                                                                    'Newly '
                                                                    'Registered '
                                                                    'Domains '
                                                                    'list with '
                                                                    'high '
                                                                    'entropy '
                                                                    '(last 30 '
                                                                    'days) to '
                                                                    'match on '
                                                                    'DNS, TLS '
                                                                    'and HTTP '
                                                                    'communication.\n'
                                                                    'Produced '
                                                                    'by Stamus '
                                                                    'Labs '
                                                                    'research '
                                                                    'team.\n',
                                                     'license': 'Commercial',
                                                     'min-version': '6.0.0',
                                                     'parameters': {   'secret-code': {   'prompt': 'Stamus '
                                                                                                    'Networks '
                                                                                                    'License '
                                                                                                    'code'}},
                                                     'subscribe-url': 'https://www.stamus-networks.com/stamus-labs/subscribe-to-threat-intel-feed',
                                                     'summary': 'Newly '
                                                                'Registered '
                                                                'Domains Open '
                                                                'only - 30 day '
                                                                'list, high '
                                                                'entropy',
                                                     'url': 'https://ti.stamus-networks.io/%(secret-code)s/sti-domains-entropy-30.tar.gz',
                                                     'vendor': 'Stamus '
                                                               'Networks'},
                   'stamus/nrd-phishing-14-open': {   'description': 'Suspicious '
                                                                     'Newly '
                                                                     'Registered '
                                                                     'Domains '
                                                                     'Phishing '
                                                                     'list '
                                                                     '(last 14 '
                                                                     'days) to '
                                                                     'match on '
                                                                     'DNS, TLS '
                                                                     'and HTTP '
                                                                     'communication.\n'
                                                                     'Produced '
                                                                     'by '
                                                                     'Stamus '
                                                                     'Labs '
                                                                     'research '
                                                                     'team.\n',
                                                      'license': 'Commercial',
                                                      'min-version': '6.0.0',
                                                      'parameters': {   'secret-code': {   'prompt': 'Stamus '
                                                                                                     'Networks '
                                                                                                     'License '
                                                                                                     'code'}},
                                                      'subscribe-url': 'https://www.stamus-networks.com/stamus-labs/subscribe-to-threat-intel-feed',
                                                      'summary': 'Newly '
                                                                 'Registered '
                                                                 'Domains Open '
                                                                 'only - 14 '
                                                                 'day list, '
                                                                 'phishing',
                                                      'url': 'https://ti.stamus-networks.io/%(secret-code)s/sti-domains-phishing-14.tar.gz',
                                                      'vendor': 'Stamus '
                                                                'Networks'},
                   'stamus/nrd-phishing-30-open': {   'description': 'Suspicious '
                                                                     'Newly '
                                                                     'Registered '
                                                                     'Domains '
                                                                     'Phishing '
                                                                     'list '
                                                                     '(last 30 '
                                                                     'days) to '
                                                                     'match on '
                                                                     'DNS, TLS '
                                                                     'and HTTP '
                                                                     'communication.\n'
                                                                     'Produced '
                                                                     'by '
                                                                     'Stamus '
                                                                     'Labs '
                                                                     'research '
                                                                     'team.\n',
                                                      'license': 'Commercial',
                                                      'min-version': '6.0.0',
                                                      'parameters': {   'secret-code': {   'prompt': 'Stamus '
                                                                                                     'Networks '
                                                                                                     'License '
                                                                                                     'code'}},
                                                      'subscribe-url': 'https://www.stamus-networks.com/stamus-labs/subscribe-to-threat-intel-feed',
                                                      'summary': 'Newly '
                                                                 'Registered '
                                                                 'Domains Open '
                                                                 'only - 30 '
                                                                 'day list, '
                                                                 'phishing',
                                                      'url': 'https://ti.stamus-networks.io/%(secret-code)s/sti-domains-phishing-30.tar.gz',
                                                      'vendor': 'Stamus '
                                                                'Networks'},
                   'tgreen/hunting': {   'checksum': False,
                                         'description': 'Heuristic ruleset for '
                                                        'hunting. Focus on '
                                                        'anomaly detection and '
                                                        'showcasing latest '
                                                        'engine features, not '
                                                        'performance.\n',
                                         'license': 'GPLv3',
                                         'min-version': '4.1.0',
                                         'summary': 'Threat hunting rules',
                                         'url': 'https://github.com/travisbgreen/hunting-rules/raw/master/hunting.rules.tar.gz',
                                         'vendor': 'tgreen'}},
    'version': 1}


# Copyright (C) 2017 Open Information Security Foundation
# Copyright (c) 2013 Jason Ish
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

from __future__ import print_function

import unittest
import tempfile

from suricata.update import util

class Md5TestCase(unittest.TestCase):

    def test_hexdigest(self):
        test_file = tempfile.NamedTemporaryFile()
        test_file.write(b"This is a test.")
        test_file.flush()
        self.assertEqual(
            "120ea8a25e5d487bf68b5f7096440019",
            util.md5_hexdigest(open(test_file.name).read().encode()))



#!/usr/bin/env python
import re
import sys
import os
from string import Template

if len(sys.argv) == 2:
    SRC_DIR = sys.argv[1]
else:
    SRC_DIR = "../../src/"


class Structure:
    def __init__(self, string):
        (self.struct, self.flags, self.values) = string.split(":")

class SetterGetter:
    def __init__(self, string):
        (function, params, self.value) = string.split(":")
        self.function = function.strip("()")
        self.params = [int(a) for a in params.split(",")]


cmd = "grep -h coccinelle ../../src/*[ch] | sed -e 's/.*coccinelle: \(.*\) \*\//\1/'"

struct_list = []
setter_getter_list = []

dirList = os.listdir(SRC_DIR)
for fname in dirList:
    if re.search("\.[ch]$", fname):
        for line in open(os.path.join(SRC_DIR, fname)):
            if "coccinelle:" in line:
                m = re.search("coccinelle: (.*) \*\/", line)
                if "()" not in m.group(1):
                    struct = Structure(m.group(1))
                    struct_list.append(struct)
                else:
                    function = SetterGetter(m.group(1))
                    setter_getter_list.append(function)

header = "@flags@"
body = []

# Handle setter and getter
setter_getter = [x.function for x in setter_getter_list]
if len(setter_getter):
    header += "\nidentifier NotSetterGetter !~ \"^(%s)$\";" % ("|".join(setter_getter))

i = 0
for struct in struct_list:
    header += """
%s *struct%d;
identifier struct_flags%d =~ "^(?!%s).+";""" % (struct.struct, i, i, struct.values)

    body.append("""
struct%d->%s@p1 |= struct_flags%d
|
struct%d->%s@p1 & struct_flags%d
|
struct%d->%s@p1 &= ~struct_flags%d
""" % (i, struct.flags, i, i, struct.flags, i, i, struct.flags, i))

    i += 1

print(header)
print("position p1;")
print("@@")
print("")
print("""
NotSetterGetter(...)
{
    <...
""")
print("")
print("(" + "|".join(body) + ")")
print("")
print("""
...>
}

@script:python@
p1 << flags.p1;
@@

print "Invalid usage of flags field at %s:%s, flags value is incorrect (wrong family)." % (p1[0].file, p1[0].line)
import sys
sys.exit(1)""")

i = 1
setter_template = """
@settergetter${i}@
identifier SetterGetter =~ "${function}";
identifier f_flags =~ "^(?!${value}).+";
identifier ${params_line};
position p1;
@@

SetterGetter@p1(${prefix_param}f_flags${suffix_param})

@script:python@
p1 << settergetter${i}.p1;
@@
print "Invalid usage of ${function} at %s:%s, flags value is incorrect (wrong family)." % (p1[0].file, p1[0].line)
import sys
sys.exit(1)
"""

for sg in setter_getter_list:
    prefix_param = ""
    for index in list(range(1, sg.params[1])):
        prefix_param += "param%d, " % (index)
    if sg.params[1] < sg.params[0]:
        suffix_param = ", " + ", ".join(["param%d" % (index + 1) for index in list(range(sg.params[1], sg.params[0]))])
    else:
        suffix_param = ""
    params_elts = list(range(1, sg.params[1])) + list(range(sg.params[1] + 1, sg.params[0] + 1))
    params_line = ", ".join(["param%d" % (x) for x in params_elts])
    print(Template(setter_template).substitute(i=i, function=sg.function, value=sg.value,
                prefix_param=prefix_param, suffix_param=suffix_param, params_line=params_line))
    i += 1



# Copyright (C) 2020 Open Information Security Foundation
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

import re
import os.path
import platform

def parse_os_release(filename="/etc/os-release"):
    os_release={}

    if not os.path.exists(filename):
        return os_release
    
    with open(filename) as fileobj:
        for line in fileobj:
            line = line.strip()
            m = re.match(r"^(\w+)=\"?(.*?)\"?$", line)
            if m:
                os_release[m.group(1)] = m.group(2)
    return os_release

def dist():
    os_release = parse_os_release()
    if "NAME" in os_release:
        version_fields = ["VERSION_ID", "BUILD_ID"]
        for vf in version_fields:
            if vf in os_release:
                return "{}/{}".format(os_release["NAME"], os_release[vf])
        return os_release["NAME"]

    # Arch may or may not have /etc/os-release, but its easy to
    # detect.
    if os.path.exists("/etc/arch-release"):
        return "Arch Linux"

    # Uname fallback.
    uname = platform.uname()
    return "{}/{}".format(uname[0], uname[2])

normalized_arch = {
    "amd64": "x86_64",
}

def arch():
    """Return the machine architecture. """
    machine = platform.machine()
    return normalized_arch.get(machine, machine)

if __name__ == "__main__":
    # Build a user agent string. Something like:
    # Suricata-Update/1.2.0dev0 (OS: Linux; \
    #    CPU: x86_64; \
    #    Python: 3.7.7; \
    #    Dist: Fedora/31; \
    #    Suricata: 4.0.0)
    parts = []
    parts.append("OS: {}".format(platform.system()))
    parts.append("CPU: {}".format(arch()))
    parts.append("Python: {}".format(platform.python_version()))
    parts.append("Dist: {}".format(dist()))

    print("Suricata-Update/1.2.0dev0 ({})".format("; ".join(parts)))



import sys
import os
import subprocess
import shutil
import tempfile
import suricata.update.rule

DATA_DIR = "./tests/tmp"


def run(args):
    subprocess.check_call(args)


def delete(path):
    if os.path.isdir(path):
        shutil.rmtree(path)
    else:
        os.unlink(path)


print("Python executable: %s" % sys.executable)
print("Python version: %s" % str(sys.version))
print("Current directory: %s" % os.getcwd())

# Override the default source index URL to avoid hitting the network.
os.environ["SOURCE_INDEX_URL"] = "file://%s/tests/index.yaml" % (os.getcwd())

os.environ["ETOPEN_URL"] = "file://%s/tests/emerging.rules.tar.gz" % (
    os.getcwd())

if os.path.exists(DATA_DIR):
    delete(DATA_DIR)

common_args = [
    sys.executable,
    "./bin/suricata-update",
    "-D",
    DATA_DIR,
    "-c",
    "./tests/empty",
]

common_update_args = [
    "--no-test",
    "--no-reload",
    "--suricata-conf",
    "./tests/suricata.yaml",
    "--disable-conf",
    "./tests/disable.conf",
    "--enable-conf",
    "./tests/empty",
    "--drop-conf",
    "./tests/empty",
    "--modify-conf",
    "./tests/empty",
]

# Default run with data directory.
run(common_args + common_update_args)
assert (os.path.exists(DATA_DIR))
assert (os.path.exists(os.path.join(DATA_DIR, "update", "cache")))
assert (os.path.exists(os.path.join(DATA_DIR, "rules", "suricata.rules")))

# Default run with data directory and --no-merge
run(common_args + common_update_args + ["--no-merge"])
assert (os.path.exists(DATA_DIR))
assert (os.path.exists(os.path.join(DATA_DIR, "update", "cache")))
assert (os.path.exists(
    os.path.join(DATA_DIR, "rules", "emerging-deleted.rules")))
assert (os.path.exists(
    os.path.join(DATA_DIR, "rules", "emerging-current_events.rules")))

# Still a default run, but set --output to an alternate location."
run(common_args + common_update_args + ["--output", "./tests/tmp/_rules"])
assert (os.path.exists(os.path.join(DATA_DIR, "_rules")))

# Update sources.
run(common_args + ["update-sources"])
assert (os.path.exists(os.path.join(DATA_DIR, "update", "cache",
                                    "index.yaml")))

# Now delete the index and run lists-sources to see if it downloads
# the index.
delete(os.path.join(DATA_DIR, "update", "cache", "index.yaml"))
run(common_args + ["list-sources"])
assert(not os.path.exists(os.path.join(DATA_DIR, "update", "cache", "index.yaml")))

# Enable a source.
run(common_args + ["enable-source", "oisf/trafficid"])
assert (os.path.exists(
    os.path.join(DATA_DIR, "update", "sources", "oisf-trafficid.yaml")))

# Disable the source.
run(common_args + ["disable-source", "oisf/trafficid"])
assert (not os.path.exists(
    os.path.join(DATA_DIR, "update", "sources", "oisf-trafficid.yaml")))
assert (os.path.exists(
    os.path.join(DATA_DIR, "update", "sources",
                 "oisf-trafficid.yaml.disabled")))

# Remove the source.
run(common_args + ["remove-source", "oisf/trafficid"])
assert (not os.path.exists(
    os.path.join(DATA_DIR, "update", "sources",
                 "oisf-trafficid.yaml.disabled")))

# Add a source with a custom header.
run(common_args + [
    "add-source", "--http-header", "Header: NoSpaces",
    "testing-header-nospaces", "file:///doesnotexist"
])

# Add a source with a custom header with spaces in the value
# (https://redmine.openinfosecfoundation.org/issues/4362)
run(common_args + [
    "add-source", "--http-header", "Authorization: Basic dXNlcjE6cGFzc3dvcmQx",
    "testing-header-with-spaces", "file:///doesnotexist"
])

run(common_args + [
    "add-source",
    "suricata-test-rules",
    "file://{}/tests/suricata-test-rules.zip".format(os.getcwd()),
])
run(common_args)
assert(os.path.exists(os.path.join(DATA_DIR, "rules/testmyids.md5")))
assert(os.path.exists(os.path.join(DATA_DIR, "rules/testmyids.sha1")))
assert(os.path.exists(os.path.join(DATA_DIR, "rules/testmyids.sha256")))

class IntegrationTest:
    def __init__(self, configs={}):
        self.directory = tempfile.mkdtemp(dir=DATA_DIR)
        self.configs = configs
        self.args = []
        self.write_configs()

        if not "update.yaml" in self.configs:
            self.args += ["-c", "./tests/empty"]

    def write_configs(self):
        for config in self.configs:
            config_filename = "%s/%s" % (self.directory, config)
            with open(config_filename, "w") as of:
                of.write(self.configs[config])
            if config == "modify.conf":
                self.args += ["--modify-conf", config_filename]
            elif config == "drop.conf":
                self.args += ["--drop-conf", config_filename]
            elif config == "enable.conf":
                self.args += ["--enable-conf", config_filename]
            elif config == "disable.conf":
                self.args += ["--disable-conf", config_filename]

    def run(self):
        args = [
            sys.executable,
            "./bin/suricata-update",
            "-D",
            self.directory,
            "--no-test",
            "--no-reload",
            "--suricata-conf",
            "./tests/suricata.yaml",
        ] + self.args
        subprocess.check_call(args)
        self.check()
        self.clean()

    def clean(self):
        if self.directory.startswith(DATA_DIR):
            shutil.rmtree(self.directory)

    def check(self):
        pass

    def get_rule_by_sid(self, sid):
        """ Return all rules where the provided substring is found. """
        with open("%s/rules/suricata.rules" % (self.directory)) as inf:
            for line in inf:
                rule = suricata.update.rule.parse(line)
                if rule.sid == sid:
                    return rule
        return None


class MultipleModifyTest(IntegrationTest):

    configs = {
        "modify.conf":
        """
modifysid emerging-exploit.rules "^alert" | "drop"
modifysid * "^drop(.*)noalert(.*)" | "alert${1}noalert${2}"
        """
    }

    def __init__(self):
        IntegrationTest.__init__(self, self.configs)

    def check(self):
        # This rule should have been converted to drop.
        rule1 = self.get_rule_by_sid(2103461)
        assert(rule1.action == "drop")

        # This one should have been converted back to alert.
        rule2 = self.get_rule_by_sid(2023184)
        assert(rule2.action == "alert")

class DropAndModifyTest(IntegrationTest):

    configs = {
        "drop.conf": """
2024029
        """,
        "modify.conf": """
2024029 "ET INFO" "TEST INFO"
        """
    }

    def __init__(self):
        IntegrationTest.__init__(self, self.configs)

    def check(self):
        rule1 = self.get_rule_by_sid(2024029)
        assert(rule1.action == "drop")
        assert(rule1.msg.startswith("TEST INFO"))


MultipleModifyTest().run()
DropAndModifyTest().run()



import re
import sys

pointer_whitelist = set([
        'use_rle_for_non_zero',
        'use_rle_for_zero',
        'self',
        'total',
])

p = re.compile("""fn[^(]+[(]((([^),]*),)*)(([^),]*)[)])""")
VARIABLE_TO_DEREFERENCE="""[A-Za-z0-9_.]+""" #+"""|(\(\*[A-Za-z0-9_.]+\))[.][A-Za-z0-9_.]+)"""
NOT_ISIZE = """([^(]|\n|(\([^i])|(\(i[^s])|(\(is[^i]))+"""
TRAILING_PARENS = """[ \t\n\)\]\}]*"""
offset_pat = VARIABLE_TO_DEREFERENCE + """[.]offset\(""" + NOT_ISIZE + """\(isize\)[ \n\t]*\)""" + TRAILING_PARENS
o2 = """\*""" + offset_pat
p2 = re.compile(o2)
pointer_cast = re.compile("""as \(\*[^\)]+\)""")
psubarray = re.compile(offset_pat)
elite_something = """if[\n ]+1337i?3?2?[\n ]*!=[\n ]*0[\n ]*({[\n ]*(SOMETHING([\n ]+'[a-zA-Z0-9_]+)?(\n )*;)[\n ]*})"""
elite_break = re.compile(elite_something.replace("SOMETHING", "break"))
elite_continue = re.compile(elite_something.replace("SOMETHING", "continue"))
def subbreak(match):
    return match.group(1)
def subfun(match):
    all_items = match.group()
    #if all_items.count(".offset") > 1:
    #    print "ignoring " + all_items +" for multi_item"
    #    assert not all_items
    #    return all_items # not the inner-most
    each_arg = all_items.split(',')
    out_arg = []
    for index in range(len(each_arg)):
        v = each_arg[index]
        out_arg.append(v)
        is_single = False
        for item in pointer_whitelist:
            if item in v:
                is_single = True
        where = v.find('*mut')
        if where != -1:
            if is_single:
                v= v.replace('*mut', '&mut', 1)
            else:
                v= v.replace('*mut', '&mut [', 1).replace('[ ','[') + ']'
        where = v.find('*const')
        if where != -1:
            if is_single:
                v= v.replace('*const', '&', 1)
            else:
                v= v.replace('*const', '& [', 1).replace('[ ','[')+']'
        if v.endswith(')]'):
            v = v[:-2] + '])'
        out_arg[index] = v
    return balance(','.join(out_arg))

def recursive_offset_match(data):
    if data.find('.offset') != -1:
        data = p2.sub(dereffun, data)
        data = psubarray.sub(desubarrayfun, data)
    return data

def zerofirstoffset(data):
    where = data.find('.offset')
    if where == -1:
        assert False
    return data[0:where] + '.\xff' + data[where + 2:]
def cut_out_both_offset(data):
    split_loc = data.find('.offset') + 1
    ret = data[split_loc:]
    ret = p2.sub(dereffun, ret)
    ret = psubarray.sub(desubarrayfun, ret)
    return data[:split_loc] + ret

def dereffun(match):
    data = match.group()
    if data.count('.offset') > 1:
        return cut_out_both_offset(data)
    data = data[1:] # zero out the *
    data = data.replace('.offset(','[(', 1)
    data = data.replace('(isize)', '(usize)')
    data = data + ']'
    data = recursive_offset_match(data)
    data = balance(data)
    return data

def desubarrayfun(match):
    data = match.group()
    if data.count('.offset') > 1:
        return cut_out_both_offset(data)
    data = data.replace('.offset(','[(', 1)
    data = data.replace('(isize)', '(usize)')
    data = data + '..]'
    data = recursive_offset_match(data)
    data = balance(data)
    return data
def balance(data):
    for ch in "[]{}()":
        data = data.replace("""b'""" + ch + """'""", str(ord(ch)))
    retlist = []
    stack = []
    bad_chars = []
    rev_paren = {
        '{':'}','}':'{',
        '[':']',']':'[',
        '(':')',')':'(',}
    while True:
        matches = [data.find(c) for c in "{}()[]"]
        where = min(x if x >= 0 else len(data) for x in matches)
        if where == len(data):
            retlist.append(data)
            data = b''
            retlist += [b.replace(' ','').replace('\n','').replace('\t','') for b in bad_chars]
            break
        ch = data[where]
        if ch in '({[':
            stack.append(ch)
            retlist.append(data[:where+1])
            data = data[where + 1:]
        else:
            if len(bad_chars) and bad_chars[0][-1] == rev_paren[ch]:
                retlist.append(bad_chars[-1])
                bad_chars = bad_chars[1:]
            elif len(stack) and ch == rev_paren[stack[-1]]:
                retlist.append(data[:where + 1])
                data = data[where+1:]
                stack.pop()
                while len(bad_chars) and len(stack) and bad_chars[0][-1] ==rev_paren[stack[-1][-1]]:
                    retlist.append(bad_chars[0])
                    bad_chars= bad_chars[1:]
                    stack.pop()
            elif len(stack) != 0:
                bad_chars.append(data[:where + 1])
                data = data[where+1:]
            else:
                retlist.append(data[:where+1])
                data = data[where+1:]
                               
    return ''.join(retlist)
def rem(match):
    return ""
with open(sys.argv[1]) as f:
    ret = p.sub(subfun, f.read())
    ret = p2.sub(dereffun, ret)
    ret = p2.sub(dereffun, ret)
    ret = p2.sub(dereffun, ret)
    ret = psubarray.sub(desubarrayfun, ret)
    ret = psubarray.sub(desubarrayfun, ret)
    ret = psubarray.sub(desubarrayfun, ret)
    ret = p2.sub(dereffun, ret)
    ret = p2.sub(dereffun, ret)
    ret = p2.sub(dereffun, ret)
    ret = ret.replace('i32 as (usize)', 'usize')
    ret = ret.replace('i32 as (u32)', 'u32')
    ret = pointer_cast.sub(rem, ret)
    ret = elite_break.sub(subbreak, ret)
    ret = elite_continue.sub(subbreak, ret)
    ret = ret.replace("#[derive(Clone, Copy)]", "")
    ret = ret.replace("#[repr(C)]", "")
    ret = ret.replace("#[no_mangle]", "")
    ret = ret.replace("unsafe extern ", "")
    ret = ret.replace("unsafe", "")
    ret = ret.replace('self', 'xself')
    #ret = balance(ret)
    sys.stdout.write(ret)



# Copyright (C) 2018 Open Information Security Foundation
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

from __future__ import print_function

import os
import io
import unittest

import suricata.update.rule
from suricata.update import main
import suricata.update.extract
from suricata.update import matchers as matchers_mod

class GroupMatcherTestCase(unittest.TestCase):

    rule_string = """alert http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,from_server; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)"""

    def test_match(self):
        rule = suricata.update.rule.parse(self.rule_string, "rules/malware.rules")
        matcher = matchers_mod.parse_rule_match("group: malware.rules")
        self.assertEqual(
            matcher.__class__, matchers_mod.GroupMatcher)
        self.assertTrue(matcher.match(rule))

        # Test match of just the group basename.
        matcher = matchers_mod.parse_rule_match("group: malware")
        self.assertEqual(
            matcher.__class__, matchers_mod.GroupMatcher)
        self.assertTrue(matcher.match(rule))

class FilenameMatcherTestCase(unittest.TestCase):

    rule_string = """alert http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,from_server; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)"""

    def test_match(self):
        rule = suricata.update.rule.parse(self.rule_string, "rules/trojan.rules")
        matcher = matchers_mod.parse_rule_match("filename: */trojan.rules")
        self.assertEqual(
            matcher.__class__, matchers_mod.FilenameMatcher)
        self.assertTrue(matcher.match(rule))

class LoadMatchersTestCase(unittest.TestCase):

    def test_trailing_comment(self):
        """Test loading matchers with a trailing comment."""
        matchers = main.parse_matchers(io.StringIO(u"""filename: */trojan.rules
re:.# This is a comment*
1:100 # Trailing comment.
"""))
        self.assertEqual(
            matchers[0].__class__, matchers_mod.FilenameMatcher)
        self.assertEqual(
            matchers[1].__class__, matchers_mod.ReRuleMatcher)
        self.assertEqual(
            matchers[2].__class__, matchers_mod.IdSetRuleMatcher)

class IdRuleMatcherTestCase(unittest.TestCase):

    def test_parse_single_sid(self):
        matcher = matchers_mod.IdRuleMatcher.parse("123")
        self.assertIsNotNone(matcher)
        self.assertEqual(1, len(matcher.signatureIds))

    def test_parse_single_gidsid(self):
        matcher = matchers_mod.IdRuleMatcher.parse("1:123")
        self.assertIsNotNone(matcher)
        self.assertEqual(1, len(matcher.signatureIds))

    def test_parse_multi_sid(self):
        matcher = matchers_mod.IdRuleMatcher.parse("1,2,3")
        self.assertIsNotNone(matcher)
        self.assertEqual(3, len(matcher.signatureIds))

    def test_parse_multi_gidsid(self):
        matcher = matchers_mod.IdRuleMatcher.parse("1:1000,2:2000,    3:3000, 4:4000")
        self.assertIsNotNone(matcher)
        self.assertEqual(4, len(matcher.signatureIds))

    def test_parse_multi_mixed(self):
        matcher = matchers_mod.IdRuleMatcher.parse("1:1000, 2000, 3:3000, 4000")
        self.assertIsNotNone(matcher)
        self.assertEqual(4, len(matcher.signatureIds))

    def test_parse_invalid(self):
        matcher = matchers_mod.IdRuleMatcher.parse("a")
        self.assertIsNone(matcher)

        matcher = matchers_mod.IdRuleMatcher.parse("1, a")
        self.assertIsNone(matcher)

        matcher = matchers_mod.IdRuleMatcher.parse("1a")
        self.assertIsNone(matcher)

        matcher = matchers_mod.IdRuleMatcher.parse("1:a")
        self.assertIsNone(matcher)

class MetadataAddTestCase(unittest.TestCase):

    def test_metadata_add(self):
        rule_string = 'alert tcp any any -> any any (msg:"SURICATA STREAM Packet is retransmission"; stream-event:pkt_retransmission; flowint:tcp.retransmission.count,+,1; noalert; classtype:protocol-command-decode; sid:2210053; rev:1;)'
        rule = suricata.update.rule.parse(rule_string)
        text = 'metadata-add re:"SURICATA STREAM" "evebox.action" "archive"'
        metadata_filter = matchers_mod.AddMetadataFilter.parse(text)
        self.assertTrue(metadata_filter.match(rule))
        new_rule = metadata_filter.run(rule)
        self.assertIsNotNone(new_rule)
        self.assertTrue(new_rule.format().find("evebox.action") > -1)

class MetadataMatchTestCase(unittest.TestCase):

    def test_match_metadata(self):
        """
        Looking for: deployment Perimeter
        """
        rule_string = b"""alert http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET WEB_SPECIFIC_APPS PHPStudy Remote Code Execution Backdoor"; flow:established,to_server; http.method; content:"GET"; http.header; content:"Accept-Charset|3a 20|"; fast_pattern; nocase; pcre:"/^(?:[A-Za-z0-9+/]{4})*(?:[A-Za-z0-9+/]{2}==|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{4})\\x0d\\x0a/R"; reference:url,www.cnblogs.com/-qing-/p/11575622.html; reference:url,www.uedbox.com/post/59265/; classtype:attempted-admin; sid:2028629; rev:1; metadata:affected_product Web_Server_Applications, attack_target Server, created_at 2019_09_25, deployment Perimeter, former_category WEB_SPECIFIC_APPS, performance_impact Significant, signature_severity Major, updated_at 2019_09_25;)"""
        rule = suricata.update.rule.parse(rule_string)
        self.assertIsNotNone(rule)
        filter_string = "metadata: deployment  perimeter"
        metadata_filter = matchers_mod.MetadataRuleMatch.parse(filter_string)
        self.assertIsNotNone(metadata_filter)
        self.assertTrue(metadata_filter.match(rule))



import sys
import os
import random as insecure_random
import subprocess
import hashlib
import traceback

def get_goal():
    ret = insecure_random.randrange(1,16)
    if ret == 15:
        ret = insecure_random.randrange(15,64)
    return ret

def shasum(work):
    sha = hashlib.sha256()
    for file in work:
        with open(file) as ff:
            while True:
                r = ff.read(65536)
                if r:
                    sha.update(r)
                else:
                    break
    return sha.digest()
def file_size(filename):
    return os.stat(filename).st_size
uncatable_stats = 0
catable_stats = 0
def process(work, brotli, cat, prefix):
    sys.stdout.flush()
    global uncatable_stats
    global catable_stats
    work = [filename for filename in work if filename.find(".jpg") != -1 or filename.find(".avi") != -1 or filename.find(".mp4") != -1 or filename.find(".mov") != -1 or insecure_random.randrange(1,17) == 16]
    if len(work) == 0:
        return
    try:
        fullsum = shasum(work)
    except Exception:
        traceback.print_exc()
        print 'ok early exit'
        return
    quality = "-q" + str(insecure_random.randrange(2,12 if len(work) < 8 else 10))
    if insecure_random.randrange(0,16) == 0:
        quality = '-q9.5'
    append = insecure_random.randrange(0,2) == 0
    frivolous_procs = []
    procs = []
    print 'processing',work,'at',quality,append
    for index, filename in enumerate(work):
        thread_count = insecure_random.randrange(1,17)
        if quality > 8 and thread_count == 1:
            thread_count = 2
        magic = insecure_random.randrange(0,2) == 0
        buffer_size = insecure_random.randrange(1,18)
        if buffer_size == 17:
            buffer_size = 65536
        elif buffer_size > 3:
            buffer_size *= 1024
        bs = "-bs" + str(buffer_size)
        args = [brotli, "-c", bs, quality]
        if magic:
            args.append("-magic")
        frivolous_procs.append(subprocess.Popen(args + [filename, prefix + quality+"-" + str(index)+".compressed"]))
        if append and index == 0:
            args.append("-appendable")
        else:
            args.append("-catable")
        args.append('-j' + str(thread_count))
        args.append(filename)
        args.append(prefix + quality+"-" + str(index)+".br")
        procs.append(subprocess.Popen(args))
    for index, proc in enumerate(procs):
        ret = proc.wait()
        if ret:
            print 'failure at ' + work[index],quality,append, bs
        assert not ret
    buffer_size = insecure_random.randrange(1,18)
    if buffer_size == 17:
        buffer_size = 65536
    elif buffer_size > 3:
        buffer_size *= 1024
    bs = "-bs" + str(buffer_size)
    args = [cat, bs]
    for index, filename in enumerate(work):
        args.append(prefix + quality+"-" + str(index)+".br")
    stdout, _stderr = subprocess.Popen(args, stdout=subprocess.PIPE).communicate()
    with open(prefix+".br", 'w') as f:
        f.write(stdout)
    procs[0] = subprocess.Popen([brotli, prefix +'.br', prefix])
    ret = procs[0].wait()
    if ret:
        print 'failure at ',work,quality,append
    assert not ret
    rtsum = shasum([prefix])
    if rtsum != fullsum:
        print 'failure at ',work,quality,append
    assert rtsum == fullsum
    print 'ok',rtsum.encode('hex')
    for (index,proc) in enumerate(frivolous_procs):
        ret = proc.wait()
        assert not ret
        uncatable_stats += file_size(prefix + quality+"-" + str(index)+".compressed")
        
    catable_stats += len(stdout)
    print uncatable_stats,'/',catable_stats,(10000*uncatable_stats/catable_stats)/100.0
    try:
        for index, filename in enumerate(work):
            os.remove(prefix + quality+"-" + str(index)+".br")
            os.remove(prefix + quality+"-" + str(index)+".compressed")
        os.remove(prefix + ".br")
        os.remove(prefix)
    except Exception:
        traceback.print_exc()
    
def main():
    start = sys.argv[1]
    brotli = sys.argv[2]
    catbrotli = sys.argv[3]
    goal = get_goal()
    work =[]
    prefix="/tmp/cat-" + os.urandom(16).encode('hex')
    for root, dirnames, filenames in os.walk(start):
        for filename in filenames:
            try:
                if file_size(os.path.join(root,filename)):
                    work.append(os.path.join(root,filename))
            except Exception:
                continue
            if len(work) >= goal:
                goal = get_goal()
                process(work, brotli, catbrotli, prefix)
                work = []
    if len(work):
        process(work, brotli, catbrotli,  prefix)
if __name__=="__main__":
    main()




# Copyright (C) 2018 Open Information Security Foundation
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.

# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

import sys
import os
import argparse
import logging

from suricata.ctl import filestore, loghandler

def init_logger():
    """ Initialize logging, use colour if on a tty. """
    if os.isatty(sys.stderr.fileno()):
        logger = logging.getLogger()
        logger.setLevel(level=logging.INFO)
        logger.addHandler(loghandler.SuriColourLogHandler())
    else:
        logging.basicConfig(
            level=logging.INFO,
            format="%(asctime)s - <%(levelname)s> - %(message)s")

def main():
    init_logger()
    parser = argparse.ArgumentParser()
    subparsers = parser.add_subparsers(help='sub-command help')
    fs_parser = subparsers.add_parser("filestore", help="Filestore related commands")
    filestore.register_args(parser=fs_parser)
    args = parser.parse_args()
    try:
        func = args.func
    except AttributeError:
        parser.error("too few arguments")
    func(args)



#!/usr/bin/python

import urllib2
from BeautifulSoup import BeautifulSoup, ResultSet
file = urllib2.urlopen('http://www.iana.org/assignments/tls-parameters/tls-parameters.xml')
data = file.read()
with open('tls-parameters.xml', 'wb') as myFile:
    myFile.write(data)
file.close()

dom = BeautifulSoup(data)

#ciphersuites=dom.findAll ("registry")[4]
ciphersuites=dom.findAll (id="tls-parameters-4")
if isinstance(ciphersuites,ResultSet):
    ciphersuites = ciphersuites.pop()

for i in ciphersuites.findAll ("record"):
  value = "".join(i.value.contents)
  desc = "".join (i.description.contents)
  
  ignore_keywords = [
          "Unassigned",
          "Reserved",
          ]
  f = filter(desc.startswith,ignore_keywords)
  
  if len(f) > 0:
    continue
  
  if desc == "TLS_EMPTY_RENEGOTIATION_INFO_SCSV":
    continue
  elif desc == "TLS_FALLBACK_SCSV":
    continue
  
  rfc = "NONE"
  if i.xref:
    rfc_tmp = filter (lambda (var,val) : var == "data", i.xref.attrs)
    if len (rfc_tmp) > 0:
      # rfc = rfc_tmp[0][1][3:7]
      rfc = rfc_tmp[0][1]

  real_value = "".join (map (lambda x : "%2.2x" % (int (x, 16)), value.split (",")))

  minver = 0x0300
  maxver = 0xffff
  
  if rfc == "rfc8446":
    kxau = ["TLS13"]
    encmac = desc[4:] # skip "TLS_"
  elif rfc == "draft-camwinget-tls-ts13-macciphersuites":
    kxau = ["TLS13"]
    encmac = "NULL_" + desc.split("_")[1] # forge string like NULL_SHA256
  else:
    (_kxau, encmac) = desc.split("_WITH_")
    kxau = _kxau.split ("_")[1:]
  export = 0
  if kxau[-1] == "EXPORT":
    export = 1
    maxver = 0x302
    kxau = kxau[:-1]
  if len (kxau) == 1:
    kx = kxau[0]
    au = kxau[0]
  elif kxau[0] == "SRP":
    kx = "_".join (kxau[0:1])
    au = kx
    if len (kxau) > 2:
      au += "+" + "_".join (kxau[2:])
  elif kxau[0] == "GOSTR341112":
    # unsupported suites from https://datatracker.ietf.org/doc/draft-smyshlyaev-tls12-gost-suites/
    continue
  else:
    kx, au = kxau
  if au == "anon":
    au = "NULL"
  
  _encmac = encmac.split ("_")
  hashfun = _encmac [-1]
  _encstr = "_".join (_encmac [:-1])
  _enc = _encmac [:-1] 
 
  if _encstr == "DES40_CBC":
    enc = "DES"
    encmode = "CBC"
    encsize = 40
  elif len (_enc) == 3 and _enc[1] == "CBC" and _enc[2] == "40":
    enc = _enc[0]
    encmode = "CBC"
    encsize = 40
  elif _encstr == "DES_CBC":
    enc = "DES"
    encmode = "CBC"
    encsize = 56
  elif _encstr == "IDEA_CBC":
    enc = "IDEA"
    encmode = "CBC"
    encsize = 128
  elif _encstr == "3DES_EDE_CBC":
    enc = "3DES"
    encmode = "CBC"
    encsize = 168
  elif _encstr == "NULL":
    enc = "NULL"
    encmode = ""
    encsize = 0
  elif _encstr == "SEED_CBC":
    enc = "SEED"
    encmode = "CBC"
    encsize = 128
  elif _encstr == "CHACHA20_POLY1305":
    enc = "CHACHA20_POLY1305"
    encmode = "CBC"
    encsize = 256
  elif len (_enc) == 2:
    enc = _enc[0]
    encmode = ""
    encsize = int (_enc[1])
  else:
    enc = _enc[0]
    encmode = _enc[2]
    encsize = int (_enc[1])

  prf = "DEFAULT"
  prfsize = 0

  # fix crap from recent changes
  if hashfun == "8":
    hashfun = "_".join([encmode,hashfun])
    encmode = ""
  
  if hashfun == "NULL":
    mac = "NULL"
    macsize = 0
  elif hashfun == "MD5":
    mac = "HMAC-MD5"
    macsize = 128
  elif hashfun == "SHA":
    mac = "HMAC-SHA1"
    macsize = 160
  elif hashfun == "SHA256":
    mac = "HMAC-SHA256"
    macsize = 256
    prf = "SHA256"
    prfsize = 256
    minver = 0x303
  elif hashfun == "SHA384":
    mac = "HMAC-SHA384"
    macsize = 384
    prf = "SHA384"
    prfsize = 384
    minver = 0x303
  elif hashfun == "CCM":
    #print encmode
    #mac = "CCM"
    #macsize = 0
    minver = 0x303
    encmode = "CCM"
  elif hashfun == "CCM_8":
    #print encmode
    #mac = "CCM_8"
    #macsize = 0
    minver = 0x303
    encmode = "CCM"
  else:
    print desc
    print encmac
    print hashfun
    raise "Unsupported."
  
  if encmode == "GCM" or encmode == "CCM":
    mac = "AEAD"
    macsize = encsize
    minver = 0x303
  if _encstr == "CHACHA20_POLY1305":
    mac = "AEAD"
    macsize = encsize
    minver = 0x303
    
  print "%s:%s:%s:%s:%s:%s:%d:%s:%d:%s:%d:%s:%d:%4.4x:%4.4x" % (real_value, desc, kx, au, enc, encmode, encsize, mac, macsize, prf, prfsize, rfc, export, minver, maxver)



# Copyright (C) 2017 Open Information Security Foundation
# Copyright (c) 2015 Jason Ish
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

from __future__ import print_function

import os
import io
import unittest

import suricata.update.rule
from suricata.update import main
import suricata.update.extract
from suricata.update import matchers as matchers_mod

class TestRulecat(unittest.TestCase):

    def test_extract_tar(self):
        files = suricata.update.extract.extract_tar(
            "tests/emerging.rules.tar.gz")
        self.assertTrue(len(files) > 0)

    def test_extract_zip(self):
        files = suricata.update.extract.extract_zip(
            "tests/emerging.rules.zip")
        self.assertTrue(len(files) > 0)

    def test_try_extract(self):
        files = suricata.update.extract.try_extract(
            "tests/emerging.rules.zip")
        self.assertTrue(len(files) > 0)

        files = suricata.update.extract.try_extract(
            "tests/emerging.rules.tar.gz")
        self.assertTrue(len(files) > 0)

        files = suricata.update.extract.try_extract(
            "tests/emerging-current_events.rules")
        self.assertIsNone(files)

class TestFetch(unittest.TestCase):

    def test_check_checksum(self):
        """Test that we detect when the checksum are the same. This is mainly
        to catch issues between Python 2 and 3.
        """
        fetch = main.Fetch()
        url = "file://%s/emerging.rules.tar.gz" % (
            os.path.dirname(os.path.realpath(__file__)))
        local_file = "%s/emerging.rules.tar.gz" % (
            os.path.dirname(os.path.realpath(__file__)))

        # The URL passed to check_checksum is actually a tuple:
        #    (url, custom-header, has checksum url)
        net_arg = (url, None, True)

        r = fetch.check_checksum(local_file, net_arg)
        self.assertTrue(r)

class ThresholdProcessorTestCase(unittest.TestCase):

    processor = main.ThresholdProcessor()

    def test_extract_regex(self):
        processor = main.ThresholdProcessor()

        line = "suppress re:java"
        self.assertEqual("java", processor.extract_regex(line))
        
        line = 'suppress re:"vulnerable java version"'
        self.assertEqual(
            "vulnerable java version", processor.extract_regex(line))

        line = "suppress re:java, track <by_src|by_dst>, ip <ip|subnet>"
        self.assertEqual("java", processor.extract_regex(line))
    
        line = 'suppress re:"vulnerable java version", track <by_src|by_dst>, ip <ip|subnet>'
        self.assertEqual(
            "vulnerable java version", processor.extract_regex(line))

        line = 'threshold re:"vulnerable java version", type threshold, track by_dst, count 1, seconds 10'
        self.assertEqual(
            "vulnerable java version", processor.extract_regex(line))

    def test_replace(self):
        rule_string = """alert http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,from_server; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)"""
        rule = suricata.update.rule.parse(rule_string)

        line = "suppress re:windows"
        self.assertEqual(
            "suppress gen_id 1, sig_id 2020757",
            self.processor.replace(line, rule))

        bad_line = "nothing to match"
        self.assertEqual(
            "nothing to match",
            self.processor.replace(bad_line, rule)
        )

        line = 'threshold re:"ET MALWARE Windows", type threshold, ' \
               'track by_dst, count 1, seconds 10'
        self.assertEqual("threshold gen_id 1, sig_id 2020757, type threshold, track by_dst, count 1, seconds 10", self.processor.replace(line, rule))

        line = 'threshold re:malware, type threshold, track by_dst, count 1, ' \
               'seconds 10'
        self.assertEqual(
            "threshold gen_id 1, sig_id 2020757, type threshold, "
            "track by_dst, count 1, seconds 10",
            self.processor.replace(line, rule))

class ModifyRuleFilterTestCase(unittest.TestCase):

    rule_string = """alert http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,from_server; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)"""

    def test_id_match(self):
        rule0 = suricata.update.rule.parse(self.rule_string)
        line = r'2020757 "\|0d 0a\|" "|ff ff|"'
        rule_filter = matchers_mod.ModifyRuleFilter.parse(line)
        self.assertTrue(rule_filter != None)
        self.assertTrue(rule_filter.match(rule0))
        rule1 = rule_filter.run(rule0)
        self.assertEqual(
            str(rule1),
            """alert http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,from_server; content:"|ff ff|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)""")

    def test_re_match(self):
        rule0 = suricata.update.rule.parse(self.rule_string)
        line = r're:classtype:trojan-activity "\|0d 0a\|" "|ff ff|"'
        rule_filter = matchers_mod.ModifyRuleFilter.parse(line)
        self.assertTrue(rule_filter != None)
        self.assertTrue(rule_filter.match(rule0))
        rule1 = rule_filter.run(rule0)
        self.assertEqual(
            str(rule1),
            """alert http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,from_server; content:"|ff ff|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)""")

    def test_re_backref_one(self):
        rule0 = suricata.update.rule.parse(self.rule_string)
        line = 're:classtype:trojan-activity "(alert)(.*)" "drop\\2"'
        rule_filter = matchers_mod.ModifyRuleFilter.parse(line)
        self.assertTrue(rule_filter != None)
        self.assertTrue(rule_filter.match(rule0))
        rule1 = rule_filter.run(rule0)
        expected = """drop http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,from_server; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)"""
        self.assertEqual(str(rule1), expected)

    def test_re_backref_two(self):
        rule0 = suricata.update.rule.parse(self.rule_string)
        line = 're:classtype:trojan-activity "(alert)(.*)(from_server)(.*)" "drop\\2to_client\\4"'
        rule_filter = matchers_mod.ModifyRuleFilter.parse(line)
        self.assertTrue(rule_filter != None)
        self.assertTrue(rule_filter.match(rule0))
        rule1 = rule_filter.run(rule0)
        expected = """drop http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,to_client; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)"""
        self.assertEqual(str(rule1), expected)

    def test_drop_to_alert(self):
        rule_in = suricata.update.rule.parse(self.rule_string)
        self.assertIsNotNone(rule_in)

        f = matchers_mod.ModifyRuleFilter.parse(
            'group:emerging-trojan.rules "^alert" "drop"')
        self.assertIsNotNone(f)

        rule_out = f.run(rule_in)
        self.assertTrue(rule_out.format().startswith("drop"))

    def test_oinkmaster_backticks(self):
        f = matchers_mod.ModifyRuleFilter.parse(
            '* "^drop(.*)noalert(.*)" "alert${1}noalert${2}"')
        rule_in ="""drop http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,to_client; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; noalert; classtype:trojan-activity; sid:2020757; rev:2;)"""
        rule_out = f.run(suricata.update.rule.parse(rule_in))
        self.assertEqual("""alert http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,to_client; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; noalert; classtype:trojan-activity; sid:2020757; rev:2;)""", rule_out.format())

    def test_oinkmaster_backticks_not_noalert(self):
        f = matchers_mod.ModifyRuleFilter.parse(
            'modifysid * "^drop(.*)noalert(.*)" | "alert${1}noalert${2}"')
        rule_in ="""drop http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,to_client; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)"""
        rule_out = f.run(suricata.update.rule.parse(rule_in))
        self.assertEqual(rule_in, rule_out.format())

    def test_oinkmaster_modify_group_name(self):
        """Test an Oinkmaster style modification line using a group name."""
        f = matchers_mod.ModifyRuleFilter.parse(
            'modifysid botcc.rules "^alert" | "drop"')
        rule_in ="""alert http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,to_client; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)"""
        rule = suricata.update.rule.parse(rule_in, "rules/botcc.rules")
        rule_out = f.run(rule)
        self.assertTrue(rule_out.format().startswith("drop"))

class DropRuleFilterTestCase(unittest.TestCase):

    rule_string = """alert http $EXTERNAL_NET any -> $HOME_NET any (msg:"ET MALWARE Windows executable sent when remote host claims to send an image 2"; flow: established,from_server; content:"|0d 0a|Content-Type|3a| image/jpeg|0d 0a 0d 0a|MZ"; fast_pattern:12,20; classtype:trojan-activity; sid:2020757; rev:2;)"""

    def test_enabled_rule(self):
        rule0 = suricata.update.rule.parse(self.rule_string, "rules/malware.rules")
        id_matcher = matchers_mod.IdRuleMatcher.parse("2020757")
        self.assertTrue(id_matcher.match(rule0))

        drop_filter = matchers_mod.DropRuleFilter(id_matcher)
        rule1 = drop_filter.run(rule0)
        self.assertEqual("drop", rule1.action)
        self.assertTrue(rule1.enabled)
        self.assertTrue(str(rule1).startswith("drop"))

    def test_disabled_rule(self):
        rule0 = suricata.update.rule.parse(
            "# " + self.rule_string, "rules/malware.rules")
        id_matcher = matchers_mod.IdRuleMatcher.parse("2020757")
        self.assertTrue(id_matcher.match(rule0))

        drop_filter = matchers_mod.DropRuleFilter(id_matcher)
        rule1 = drop_filter.run(rule0)
        self.assertEqual("drop", rule1.action)
        self.assertFalse(rule1.enabled)
        self.assertTrue(str(rule1).startswith("# drop"))
        
    def test_drop_noalert(self):
        """ Test the rules with "noalert" are not marked as drop. """

        rule_without_noalert = """alert tcp $HOME_NET any -> $EXTERNAL_NET any (msg:"ET TROJAN [CrowdStrike] ANCHOR PANDA Torn RAT Beacon Message Header Local"; flow:established, to_server; dsize:16; content:"|00 00 00 11 c8 00 00 00 00 00 00 00 00 00 00 00|"; depth:16; flowbits:set,ET.Torn.toread_header; reference:url,blog.crowdstrike.com/whois-anchor-panda/index.html; classtype:trojan-activity; sid:2016659; rev:2; metadata:created_at 2013_03_22, updated_at 2013_03_22;)"""

        rule_with_noalert = """alert tcp $HOME_NET any -> $EXTERNAL_NET any (msg:"ET TROJAN [CrowdStrike] ANCHOR PANDA Torn RAT Beacon Message Header Local"; flow:established, to_server; dsize:16; content:"|00 00 00 11 c8 00 00 00 00 00 00 00 00 00 00 00|"; depth:16; flowbits:set,ET.Torn.toread_header; flowbits: noalert; reference:url,blog.crowdstrike.com/whois-anchor-panda/index.html; classtype:trojan-activity; sid:2016659; rev:2; metadata:created_at 2013_03_22, updated_at 2013_03_22;)"""

        rule = suricata.update.rule.parse(rule_without_noalert)
        matcher = matchers_mod.IdRuleMatcher.parse("2016659")
        rule_filter = matchers_mod.DropRuleFilter(matcher)
        self.assertTrue(rule_filter.match(rule))

        rule = suricata.update.rule.parse(rule_with_noalert)
        matcher = matchers_mod.IdRuleMatcher.parse("2016659")
        rule_filter = matchers_mod.DropRuleFilter(matcher)
        self.assertFalse(rule_filter.match(rule))


class DummySuriConf(dict):
    def __getattr__(self, val):
        return self[val]


class ClassificationConfigMergeTestCase(unittest.TestCase):
    test_fname1 = "tests/classification1.config"
    test_fname2 = "tests/classification2.config"

    def test_merge_classification_files(self):
        """ Test if the two files get merged properly and priority is maintained"""
        suriconf = DummySuriConf()
        suriconf["build_info"] = {}
        with open(self.test_fname1) as fp:
            test_file1 = fp.read()
        with open(self.test_fname2) as fp:
            test_file2 = fp.read()
        files = [("test_file1", test_file1.encode()),
                ("test_file2", test_file2.encode())]
        cdict = main.load_classification(suriconf, files)

        # Number of classifications in classification1.config: 42
        # Number of classifications in classification2.config: 44 (2 new)
        self.assertEqual(44, len(cdict))

        # classification1.config:
        # config classification: misc-attack,Misc Attack,2
        #
        # classification2.config:
        # config classification: misc-attack,Misc Attack,5
        self.assertEqual("5", cdict["misc-attack"][1])



# Copyright (C) 2018 Open Information Security Foundation
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

from __future__ import print_function

import textwrap

# Address group notes.
address_group_vars = set()

# Port group notes.
port_group_vars = set()

# Template for missing address-group variable.
missing_address_group_var_template = """ 
A rule has been disabled due to the unknown address-group variable
%(var)s being used. You may want to add this variable to your Suricata
configuration file.
"""

# Template for missing port-group variable.
missing_port_group_var_template = """ 
A rule has been disabled due to the unknown port-group variable
%(var)s being used. You may want to add this variable to your Suricata
configuration file.
"""

def render_note(note):
    lines = textwrap.wrap(note.strip().replace("\n", " "))
    print("* %s" % (lines[0]))
    for line in lines[1:]:
        print("  %s" % (line))

def dump_notes():
    notes = []

    for var in address_group_vars:
        notes.append(missing_address_group_var_template % {"var": var})

    for var in port_group_vars:
        notes.append(missing_port_group_var_template % {"var": var})

    if notes:
        print("\nNotes:\n")
        for note in notes:
            render_note(note)
            print("")



argsd = {
    "pcap-file": [
        {
            "name": "filename",
            "required": 1,
        },
        {
            "name": "output-dir",
            "required": 1,
        },
        {
            "name": "tenant",
            "type": int,
            "required": 0,
        },
        {
            "name": "continuous",
            "required": 0,
        },
        {
            "name": "delete-when-done",
            "required": 0,
        },
    ],
    "pcap-file-continuous": [
        {
            "name": "filename",
            "required": 1,
        },
        {
            "name": "output-dir",
            "required": 1,
        },
        {
            "name": "continuous",
            "val": True,
            "required": 1,
        },
        {
            "name": "tenant",
            "type": int,
            "required": 0,
        },
        {
            "name": "delete-when-done",
            "required": 0,
        },
    ],
    "iface-stat": [
        {
            "name": "iface",
            "required": 1,
        },
    ],
    "conf-get": [
        {
            "name": "variable",
            "required": 1,
        }
    ],
    "unregister-tenant-handler": [
        {
            "name": "id",
            "type": int,
            "required": 1,
        },
        {
            "name": "htype",
            "required": 1,
        },
        {
            "name": "hargs",
            "type": int,
            "required": 0,
        },
    ],
    "register-tenant-handler": [
        {
            "name": "id",
            "type": int,
            "required": 1,
        },
        {
            "name": "htype",
            "required": 1,
        },
        {
            "name": "hargs",
            "type": int,
            "required": 0,
        },
    ],
    "unregister-tenant": [
        {
            "name": "id",
            "type": int,
            "required": 1,
        },
    ],
    "register-tenant": [
        {
            "name": "id",
            "type": int,
            "required": 1,
        },
        {
            "name": "filename",
            "required": 1,
        },
    ],
    "reload-tenant": [
        {
            "name": "id",
            "type": int,
            "required": 1,
        },
        {
            "name": "filename",
            "required": 0,
        },
    ],
    "add-hostbit": [
        {
            "name": "ipaddress",
            "required": 1,
        },
        {
            "name": "hostbit",
            "required": 1,
        },
        {
            "name": "expire",
            "type": int,
            "required": 1,
        },
    ],
    "remove-hostbit": [
        {
            "name": "ipaddress",
            "required": 1,
        },
        {
            "name": "hostbit",
            "required": 1,
        },
    ],
    "list-hostbit": [
        {
            "name": "ipaddress",
            "required": 1,
        },
    ],
    "memcap-set": [
        {
            "name": "config",
            "required": 1,
        },
        {
            "name": "memcap",
            "required": 1,
        },
    ],
    "memcap-show": [
        {
            "name": "config",
            "required": 1,
        },
    ],
    "dataset-add": [
        {
            "name": "setname",
            "required": 1,
        },
        {
            "name": "settype",
            "required": 1,
        },
        {
            "name": "datavalue",
            "required": 1,
        },
    ],
    "dataset-remove": [
        {
            "name": "setname",
            "required": 1,
        },
        {
            "name": "settype",
            "required": 1,
        },
        {
            "name": "datavalue",
            "required": 1,
        },
    ],
    "get-flow-stats-by-id": [
        {
            "name": "flow_id",
            "type": int,
            "required": 1,
        },
    ],
    "dataset-clear": [
        {
            "name": "setname",
            "required": 1,
        },
        {
            "name": "settype",
            "required": 1,
        }
    ],
    "dataset-lookup": [
        {
            "name": "setname",
            "required": 1,
        },
        {
            "name": "settype",
            "required": 1,
        },
        {
            "name": "datavalue",
            "required": 1,
        },
    ],
    }



# Copyright (C) 2017-2019 Open Information Security Foundation
# Copyright (c) 2011 Jason Ish
#
# You can copy, redistribute or modify this Program under the terms of
# the GNU General Public License version 2 as published by the Free
# Software Foundation.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# version 2 along with this program; if not, write to the Free Software
# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
# 02110-1301, USA.

""" Module for parsing Snort-like rules.

Parsing is done using regular expressions and the job of this module
is to do its best at parsing out fields of interest from the rule
rather than perform a sanity check.

The methods that parse multiple rules for a provided input
(parse_file, parse_fileobj) return a list of rules instead of dict
keyed by ID as its not the job of this module to detect or deal with
duplicate signature IDs.
"""

from __future__ import print_function

import sys
import re
import logging
import io

logger = logging.getLogger(__name__)

# Compile an re pattern for basic rule matching.
rule_pattern = re.compile(r"^(?P<enabled>#)*[\s#]*"
                r"(?P<raw>"
                r"(?P<header>[^()]+)"
                r"\((?P<options>.*)\)"
                r"$)")

# Rule actions we expect to see.
actions = (
    "alert", "log", "pass", "activate", "dynamic", "drop", "reject", "sdrop")

class NoEndOfOptionError(Exception):
    """Exception raised when the end of option terminator (semicolon) is
    missing."""
    pass

class Rule(dict):
    """Class representing a rule.

    The Rule class is a class that also acts like a dictionary.

    Dictionary fields:

    - **group**: The group the rule belongs to, typically the filename.
    - **enabled**: True if rule is enabled (uncommented), False is
      disabled (commented)
    - **action**: The action of the rule (alert, pass, etc) as a
      string
    - **proto**: The protocol of the rule.
    - **direction**: The direction string of the rule.
    - **gid**: The gid of the rule as an integer
    - **sid**: The sid of the rule as an integer
    - **rev**: The revision of the rule as an integer
    - **msg**: The rule message as a string
    - **flowbits**: List of flowbit options in the rule
    - **metadata**: Metadata values as a list
    - **references**: References as a list
    - **classtype**: The classification type
    - **priority**: The rule priority, 0 if not provided
    - **noalert**: Is the rule a noalert rule
    - **features**: Features required by this rule
    - **raw**: The raw rule as read from the file or buffer

    :param enabled: Optional parameter to set the enabled state of the rule
    :param action: Optional parameter to set the action of the rule
    :param group: Optional parameter to set the group (filename) of the rule

    """

    def __init__(self, enabled=None, action=None, group=None):
        dict.__init__(self)
        self["enabled"] = enabled
        self["action"] = action
        self["proto"] = None
        self["source_addr"] = None
        self["source_port"] = None
        self["direction"] = None
        self["dest_addr"] = None
        self["dest_port"] = None
        self["group"] = group
        self["gid"] = 1
        self["sid"] = None
        self["rev"] = 0
        self["msg"] = None
        self["flowbits"] = []
        self["metadata"] = []
        self["references"] = []
        self["classtype"] = None
        self["priority"] = 0
        self["noalert"] = False

        self["features"] = []

        self["raw"] = None

    def __getattr__(self, name):
        return self[name]

    @property
    def id(self):
        """ The ID of the rule.

        :returns: A tuple (gid, sid) representing the ID of the rule
        :rtype: A tuple of 2 ints
        """
        return (int(self.gid), int(self.sid))

    @property
    def idstr(self):
        """Return the gid and sid of the rule as a string formatted like:
        '[GID:SID]'"""
        return "[%s:%s]" % (str(self.gid), str(self.sid))

    def brief(self):
        """ A brief description of the rule.

        :returns: A brief description of the rule
        :rtype: string
        """
        return "%s[%d:%d] %s" % (
            "" if self.enabled else "# ", self.gid, self.sid, self.msg)

    def __hash__(self):
        return self["raw"].__hash__()

    def __str__(self):
        """ The string representation of the rule.

        If the rule is disabled it will be returned as commented out.
        """
        return self.format()

    def format(self):
        if self.noalert and not "noalert;" in self.raw:
            self.raw = re.sub(r'( *sid\: *[0-9]+\;)', r' noalert;\1', self.raw)
        return u"%s%s" % (u"" if self.enabled else u"# ", self.raw)

def find_opt_end(options):
    """ Find the end of an option (;) handling escapes. """
    offset = 0

    while True:
        i = options[offset:].find(";")
        if options[offset + i - 1] == "\\":
            offset += 2
        else:
            return offset + i

class BadSidError(Exception):
    """Raises exception when sid is of type null"""

def parse(buf, group=None):
    """ Parse a single rule for a string buffer.

    :param buf: A string buffer containing a single Snort-like rule

    :returns: An instance of of :py:class:`.Rule` representing the parsed rule
    """

    if type(buf) == type(b""):
        buf = buf.decode("utf-8")
    buf = buf.strip()

    m = rule_pattern.match(buf)
    if not m:
        return None

    if m.group("enabled") == "#":
        enabled = False
    else:
        enabled = True

    header = m.group("header").strip()

    rule = Rule(enabled=enabled, group=group)

    # If a decoder rule, the header will be one word.
    if len(header.split(" ")) == 1:
        action = header
        direction = None
    else:
        states = ["action",
                  "proto",
                  "source_addr",
                  "source_port",
                  "direction",
                  "dest_addr",
                  "dest_port",
                  ]
        state = 0

        rem = header
        while state < len(states):
            if not rem:
                return None
            if rem[0] == "[":
                end = rem.find("]")
                if end < 0:
                    return
                end += 1
                token = rem[:end].strip()
                rem = rem[end:].strip()
            else:
                end = rem.find(" ")
                if end < 0:
                    token = rem
                    rem = ""
                else:
                    token = rem[:end].strip()
                    rem = rem[end:].strip()

            if states[state] == "action":
                action = token
            elif states[state] == "proto":
                rule["proto"] = token
            elif states[state] == "source_addr":
                rule["source_addr"] = token
            elif states[state] == "source_port":
                rule["source_port"] = token
            elif states[state] == "direction":
                direction = token
            elif states[state] == "dest_addr":
                rule["dest_addr"] = token
            elif states[state] == "dest_port":
                rule["dest_port"] = token

            state += 1

    if action not in actions:
        return None

    rule["action"] = action
    rule["direction"] = direction
    rule["header"] = header

    options = m.group("options")

    while True:
        if not options:
            break
        index = find_opt_end(options)
        if index < 0:
            raise NoEndOfOptionError("no end of option")
        option = options[:index].strip()
        options = options[index + 1:].strip()

        if option.find(":") > -1:
            name, val = [x.strip() for x in option.split(":", 1)]
        else:
            name = option
            val = None

        if name in ["gid", "sid", "rev"]:
            rule[name] = int(val)
        elif name == "metadata":
            if not name in rule:
                rule[name] = []
            rule[name] += [v.strip() for v in val.split(",")]
        elif name == "flowbits":
            rule.flowbits.append(val)
            if val and val.find("noalert") > -1:
                rule["noalert"] = True
        elif name == "noalert":
            rule["noalert"] = True
        elif name == "reference":
            rule.references.append(val)
        elif name == "msg":
            if val and val.startswith('"') and val.endswith('"'):
                val = val[1:-1]
            rule[name] = val
        else:
            rule[name] = val

        if name.startswith("ja3"):
            rule["features"].append("ja3")

    if rule["msg"] is None:
        rule["msg"] = ""

    if not rule["sid"]:
        raise BadSidError("Sid cannot be of type null")

    rule["raw"] = m.group("raw").strip()

    return rule

def parse_fileobj(fileobj, group=None):
    """ Parse multiple rules from a file like object.

    Note: At this time rules must exist on one line.

    :param fileobj: A file like object to parse rules from.

    :returns: A list of :py:class:`.Rule` instances, one for each rule parsed
    """
    rules = []
    buf = ""
    for line in fileobj:
        try:
            if type(line) == type(b""):
                line = line.decode()
        except:
            pass
        if line.rstrip().endswith("\\"):
            buf = "%s%s " % (buf, line.rstrip()[0:-1])
            continue
        buf = buf + line
        try:
            rule = parse(buf, group)
            if rule:
                rules.append(rule)
        except Exception as err:
            logger.error("Failed to parse rule: %s: %s", buf.rstrip(), err)
        buf = ""
    return rules

def parse_file(filename, group=None):
    """ Parse multiple rules from the provided filename.

    :param filename: Name of file to parse rules from

    :returns: A list of :py:class:`.Rule` instances, one for each rule parsed
    """
    with io.open(filename, encoding="utf-8") as fileobj:
        return parse_fileobj(fileobj, group)

class FlowbitResolver(object):

    setters = ["set", "setx", "unset", "toggle"]
    getters = ["isset", "isnotset"]

    def __init__(self):
        self.enabled = []

    def resolve(self, rules):
        required = self.get_required_flowbits(rules)
        enabled = self.set_required_flowbits(rules, required)
        if enabled:
            self.enabled += enabled
            return self.resolve(rules)
        return self.enabled

    def set_required_flowbits(self, rules, required):
        enabled = []
        for rule in [rule for rule in rules.values() if not rule.enabled]:
            for option, value in map(self.parse_flowbit, rule.flowbits):
                if option in self.setters and value in required:
                    rule.enabled = True
                    enabled.append(rule)
        return enabled

    def get_required_rules(self, rulemap, flowbits, include_enabled=False):
        """Returns a list of rules that need to be enabled in order to satisfy
        the list of required flowbits.

        """
        required = []

        for rule in [rule for rule in rulemap.values()]:
            if not rule:
                continue
            for option, value in map(self.parse_flowbit, rule.flowbits):
                if option in self.setters and value in flowbits:
                    if rule.enabled and not include_enabled:
                        continue
                    required.append(rule)

        return required

    def get_required_flowbits(self, rules):
        required_flowbits = set()
        for rule in [rule for rule in rules.values() if rule and rule.enabled]:
            for option, value in map(self.parse_flowbit, rule.flowbits):
                if option in self.getters:
                    required_flowbits.add(value)
        return required_flowbits

    def parse_flowbit(self, flowbit):
        tokens = flowbit.split(",", 1)
        if len(tokens) == 1:
            return tokens[0], None
        elif len(tokens) == 2:
            return tokens[0], tokens[1]
        else:
            raise Exception("Flowbit parse error on %s" % (flowbit))

def enable_flowbit_dependencies(rulemap):
    """Helper function to resolve flowbits, wrapping the FlowbitResolver
    class. """
    resolver = FlowbitResolver()
    return resolver.resolve(rulemap)

def format_sidmsgmap(rule):
    """ Format a rule as a sid-msg.map entry. """
    try:
        return " || ".join([str(rule.sid), rule.msg] + rule.references)
    except:
        logger.error("Failed to format rule as sid-msg.map: %s" % (str(rule)))
        return None

def format_sidmsgmap_v2(rule):
    """ Format a rule as a v2 sid-msg.map entry.

    eg:
    gid || sid || rev || classification || priority || msg || ref0 || refN
    """
    try:
        return " || ".join([
            str(rule.gid), str(rule.sid), str(rule.rev),
            "NOCLASS" if rule.classtype is None else rule.classtype,
            str(rule.priority), rule.msg] + rule.references)
    except:
        logger.error("Failed to format rule as sid-msg-v2.map: %s" % (
            str(rule)))
        return None

def parse_var_names(var):
    """ Parse out the variable names from a string. """
    if var is None:
        return []
    return re.findall(r"\$([\w_]+)", var)
